#!/www/python/bin/python
"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/test/test_category.py $
$Id: test_category.py 22652 2003-09-30 14:11:54Z dbinger $

Test for dulcinea.category.
"""

from sancho.unittest import TestScenario, parse_args, run_scenarios
from dulcinea.category import Category, Grouped

tested_modules = [ "dulcinea.category" ]

class CategoryTest (TestScenario):

    def check_basic (self):
        self.test_stmt('self.category = Category()')
        self.test_val('str(self.category)', '*unnamed*')
        self.test_false('self.category.check_name("ab%")')
        self.test_false('self.category.check_name("[ab]")')
        self.test_false('self.category.check_name("a\b")')
        self.test_false('self.category.check_name("a b")')
        self.test_false('self.category.check_name("a/b")')
        self.test_false('self.category.check_name(" a/b")')
        self.test_true('self.category.check_name("__")')
        self.test_true('self.category.check_name("34")')
        self.test_true('self.category.check_name("a8")')
        self.test_true('self.category.check_name("_a8")')
        self.test_exc('self.category.set_name(" ")', ValueError)
        self.test_stmt('self.category.set_name("boo")')
        self.test_val('self.category.get_name()', "boo")
        self.test_stmt('self.category.set_label("Boo")')
        self.test_val('self.category.get_label()', "Boo")
        self.test_stmt('self.category.set_description("casual")')
        self.test_val('self.category.get_description()', "casual")

    def check_grouped (self):
        self.test_stmt('self.grouped = Grouped()')
        self.test_stmt('self.category1 = Category()')
        self.test_stmt('self.category2 = Category()')
        self.test_exc('self.grouped.add_group(None)', TypeError)
        self.test_exc('self.grouped.add_group(42)', TypeError)
        self.test_stmt('self.grouped.add_group(self.category1)')
        self.test_true('self.grouped.in_group(self.category1)')
        self.test_false('self.grouped.in_group(None)')
        self.test_true('self.grouped.in_groups([self.category1])')
        self.test_set('self.grouped.get_groups()', [self.category1])
        self.test_set('self.grouped.get_descendants_of(self.category1)',
                      [self.category1])
        self.test_set('self.grouped.get_descendants_of(self.category2)',
                      [])
        self.test_false(
            'self.grouped.in_groups([self.category1, self.category2])')
        
    def check_ancestry (self):
        self.test_stmt('self.a = Category("a")')
        self.test_stmt('self.b = Category("b")')
        self.test_stmt('self.c = Category("c")')
        self.test_stmt('self.d = Category("d")')
        self.test_set('self.a.get_children()', [])
        self.test_set('self.b.get_parents()', [])
        self.test_stmt('self.a.add_child(self.b)')
        self.test_set('self.a.get_children()', [self.b])
        self.test_set('self.b.get_parents()', [self.a])
        self.test_set('self.a.get_children()', [self.b])
        self.test_set('self.b.get_parents()', [self.a])
        self.test_exc('self.b.add_child(self.a)', ValueError)
        self.test_stmt('self.b.add_child(self.c)')
        self.test_set('self.c.get_children()', [])
        self.test_set('self.c.get_parents()', [self.b])
        # Now we have a -> b -> c.
        self.test_set('self.a.get_descendants()',[self.b, self.c])
        self.test_set('self.b.get_descendants()',[self.c])
        self.test_set('self.c.get_descendants()',[])
        self.test_true('self.c.is_descendant_of(self.a)')
        self.test_true('self.c.is_descendant_of(self.b)')
        self.test_true('self.c.is_descendant_of(self.c)')
        self.test_true('self.c.is_ancestor_of(self.c)')
        self.test_false('self.d.is_ancestor_of(self.c)')
        self.test_stmt('self.a.set_children([self.b, self.c])')
        # a -> b, a-> c, b-> c, d-> b, d-> c
        self.test_true('self.c.is_descendant_of(self.a)')
        self.test_true('self.c.is_descendant_of(self.b)')
        self.test_true('self.c.is_descendant_of(self.c)')
        self.test_exc('self.a.set_parents([self.b, self.c])', ValueError)
        self.test_true('self.c.is_descendant_of(self.a)')
        self.test_true('self.c.is_descendant_of(self.b)')
        self.test_true('self.c.is_descendant_of(self.c)')
        self.test_stmt('self.a.set_parents([self.d])')
        self.test_true('self.c.is_descendant_of(self.d)')
        self.test_stmt('self.c.set_parents(list(self.c.get_parents()))')
        self.test_set('list(self.a.expand())', [self.a, self.b, self.c])
        self.test_set('list(self.b.expand())', [self.b, self.c])
        self.test_set('list(self.c.expand())', [self.c])
        self.test_set('list(self.d.expand())',
                      [self.a, self.b, self.c, self.d])
        self.test_stmt('self.a.remove_child(self.c)')
        self.test_true('self.a.is_ancestor_of(self.c)')
        self.test_stmt('self.b.remove_child(self.c)')
        self.test_false('self.a.is_ancestor_of(self.c)')
        self.test_set('list(self.b.expand())', [self.b])
        self.test_set('list(self.a.expand())', [self.a, self.b])
        self.test_stmt('self.a.get_leaves()')

if __name__ == "__main__":
    (scenarios, options) = parse_args()
    run_scenarios (scenarios, options)




"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/sendmail.py $
$Id: sendmail.py 22816 2003-10-22 23:23:34Z rmasse $

General interface for sending mail to users.
"""

from quixote.sendmail import sendmail as qx_sendmail
from quixote.sendmail import RFC822Mailbox
from dulcinea import local

def sendmail (subject, msg_body, to_addrs,
              from_addr=None, reply_to=None, smtp_sender=None,
              cc_addrs=None, extra_headers=None,
              smtp_recipients=None, force=False):
    """
    Wrapper for Quixote's sendmail() function.  Differences:
      * respects the local.SUPPRESS_EMAIL config setting
        except when the force keyword argument is true
      * always adds "Precedence", "X-No-Archive" headers
      * optionally adds "Reply-To" header
      * if smtp_sender not supplied, defaults to MAIL_SMTP_SENDER
        defined in local module rather than the addr-spec
        in from_addr

    See quixote.sendmail module for details (including argument types).
    """
    if not force and local.SUPPRESS_EMAIL:
        return

    # Control over the actual (SMTP) recipient of this message is now in
    # the hands of Quixote's sendmail(), guided by the mail_debug_addr
    # setting in the publisher's config.

    if extra_headers is None:
        extra_headers = []
    if (len(to_addrs) == 1 and
        type(to_addrs[0]) is str and
        to_addrs[0].startswith("webmaster@")):
        # Looks like this mail was generated by our feedback form.
        # Don't put in the "bulk mail" headers, because they suppress
        # the webmaster autoresponder running on kronos.
        pass
    else:
        # Add these "bulk mail" headers for all other site-generated mail.
        # They're generally nice to put in auto-generated email.
        extra_headers.extend(["Precedence: junk", "X-No-Archive: yes"])

    if reply_to is not None:
        reply_to = [RFC822Mailbox(args).format() for args in reply_to]
        extra_headers.append("Reply-To: %s" % ", ".join(reply_to))

    if smtp_sender is None:
        smtp_sender=local.MAIL_SMTP_SENDER
    qx_sendmail(subject, msg_body, to_addrs,
                from_addr=from_addr, extra_headers=extra_headers,
                cc_addrs=cc_addrs, smtp_sender=smtp_sender,
                smtp_recipients=smtp_recipients)

def format_email(user):
    """(user : DulcineaUser) -> (string, string)
    Return an e-mail, full name tuple that can be passed to
    our sendmail() function.
    """
    return user.get_email(), user.format_realname() or ''




"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/user.py $
$Id: user.py 23216 2003-11-21 21:00:24Z dbinger $

Provides a basic DulcineaUser class (user ID and password hash only)
along with tools for hashing and generating passwords as well as
a more heavyweight Contact class that uses the PermissionManager to
determine admin capability and has an Address associated with it
"""

import re, sha, time, binascii
from dulcinea import local
from dulcinea.address import Addressable
from dulcinea.base import DulcineaPersistent
from dulcinea.typeutils import typecheck


def hash_password(password):
    """Apply a one way hash function to a password and return the result."""
    return sha.new(password).hexdigest()


class PasswordGenerator:
    """
    Instance attributes:
      hash : any
    """
    default_seed = "iGcnvxFfyniL2w" # some random secret data

    def __init__(self, seed=default_seed):
        self.hash = sha.new(seed)
        self.hash.update(str(time.time()))

    def generate(self, seed="", length=6):
        """Generate a password.  Some effort is made to make it random.

        seed: if supplied the str() of this argument is used to update the
              generator state before generating the password.

        length: the maximum length of the password to return.

        """
        try:
            # Try to use /dev/urandom.
            self.hash.update(open("/dev/urandom", "rb").read(length))
        except IOError:
            # No source of random data.  This method will now only
            # generate passwords that look random if seed is kept secret.
            self.hash.update(str(time.time()))
        self.hash.update(str(seed))
        return binascii.b2a_base64(self.hash.digest())[:length]

_password_generator = PasswordGenerator()


class DulcineaUser(DulcineaPersistent):
    """Information for a registered user.

    Instance attributes:
      id : string
        unique identifier for this user
      password_hash : string
        the hashed version of the user's password, created using
        the hash_password function
    """

    user_id_re = re.compile('^[-A-Za-z0-9_@.]*$')

    def __init__(self, user_id=None):
        self.id = None
        if user_id is not None:
            self.set_id(user_id)

        self.password_hash = None


    def __str__(self):
        return self.id or "*no id*"

    format = format_realname = __str__ # subclasses should override

    # -- User ID methods -----------------------------------------------

    def set_id(self, user_id):
        typecheck(user_id, str)
        assert self.id is None, "'id' may only be set once"
        if not self.user_id_re.match(user_id):
            raise ValueError(
                'Invalid user ID %r: can only contain '
                'letters, numbers, and "-_@."' % user_id)
        self.id = user_id

    def get_id(self):
        return self.id


    # -- Password methods ----------------------------------------------

    def set_password(self, new_password):
        """Set the user's password to 'new_password'."""
        self.password_hash = hash_password(new_password)

    def valid_password(self, password):
        """Return true if the provided password is correct."""
        return self.password_hash == hash_password(password)

    def generate_password (self):
        """Set the password to a random value and return the new password."""
        seed = self.id + str(self.password_hash)
        password = _password_generator.generate(seed)
        self.set_password(password)
        return password

    def get_email(self):
        return None

    def format_realname(self):
        return ''

    def is_admin(self):
        """Override if you want superiority."""
        return False


class Contact(DulcineaUser, Addressable):
    """A more heavyweight user object that contains an Address object
    and relys on a PermissionManager to determine if the user has admin power

    Instance attributes:
      email : string
        the user's Internet email address
    """
    def __init__(self, user_id=None):
        DulcineaUser.__init__(self, user_id=user_id)
        Addressable.__init__(self)
        self.email = None

    def format_realname(self):
        return (self.address and self.address.contact_name) or ''

    def set_email(self, email):
        self.email = email

    def get_email(self):
        return self.email

    def is_admin(self):
        return local.get_user_database().get_admin().directly_grants(
            'staff', self)



"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/permission.py $
$Id: permission.py 23445 2004-02-06 16:14:55Z dbinger $

"""

from types import NoneType, DictType
from dulcinea.base import PersistentMapping
from dulcinea.persistent_list import PersistentList
from dulcinea.typeutils import typecheck
from dulcinea.spec import mapping, no, instance, sequence

class PermissionManager:

    """
    Class attributes:
      valid_permissions : { permission:string : description:string }
        Subclasses should set this value to list and describe the permissions
        they may grant.

    Instance attributes:
      permissions : PersistentMapping
        This mapping records the circumstances under which specific permissions
        are granted.
        If permissions['happy'].items()
             includes (other_permission_manager, ['rich']),
        it means that this permission manager grants 'happy' to any object
        for which the other_permission_manager grants 'rich'.
        If permissions['happy'].items() includes
          (non_permission_manager, None),
        it means that this permission manager grants 'happy' to
        non_permission_manager, with no other prerequisite.

    """

    permissions_is = mapping({str: mapping({
        instance('PermissionManager'): sequence(str, PersistentList),
        no(instance('PermissionManager')): None }, PersistentMapping)},
                             PersistentMapping)

    valid_permissions = None

    def __init__(self):
        self.clear_permissions()

    def clear_permissions(self):
        self.permissions = PersistentMapping()
        for permission in self.valid_permissions:
            self.permissions[permission] = PersistentMapping()

    def set_permission(self, permission, object, prerequisite=None):
        typecheck(permission, str)
        if prerequisite is None:
            assert not isinstance(object, PermissionManager)
            self.permissions[permission][object] = None
            return

        typecheck(prerequisite, str)
        typecheck(object, PermissionManager)

        if not self.permissions[permission].has_key(object):
            self.permissions[permission][object] = PersistentList()

        prerequisites = self.permissions[permission][object]
        if prerequisite not in prerequisites:
            prerequisites.append(prerequisite)

    def set_permission_items(self, permission, items):
        self.permissions[permission].clear()
        for object, prerequisite in items:
            self.set_permission(permission, object, prerequisite=prerequisite)

    def remove_permission(self, permission, object):
        typecheck(permission, str)
        del self.permissions[permission][object]

    def get_valid_permissions(self):
        return self.valid_permissions

    def get_permission_items(self, permission):
        for object, prerequisites in self.permissions[permission].items():
            if prerequisites is None:
                yield (object, None)
            else:
                for prerequisite in prerequisites:
                    yield (object, prerequisite)

    def get_direct_grantees(self, permission):
        return [ grantee
                 for grantee, prerequisite
                 in self.get_permission_items(permission)
                 if prerequisite is None ]

    def get_permissions(self, object):
        return [ permission
                 for permission in self.permissions.keys()
                 if self.grants(permission, object) ]

    def directly_grants(self, permission, object):
        """(permission:str, object)
            -> boolean

        Test if permission is granted directly to object.
        This is much faster than grants(), and often sufficient.
        """
        return self.permissions.get(permission, {}).get(object, 1) is None

    def grants(self, permission, object, _covered=None):
        """(permission:str, object, _covered:dict=None)
            -> None|[(str,PermissionManager)+]

        Test if permission is granted to object, returning the
        list of (permission, PermissionManager) pairs that explains
        a positive result.  Return None if the permission is
        not granted.
        """
        if permission not in self.valid_permissions:
            return None
        assert type(_covered) in (NoneType, DictType)
        # If (object, None) is an item in self.permissions[permission].
        if self.permissions[permission].get(object, 1) is None:
            # This list explains that permission is granted by self.
            return [(permission, self)]
        else:
            covered = _covered or {}
            if (permission, self) in covered:
                return None
            for permission_manager, prerequisite in self.get_permission_items(
                permission):
                if prerequisite is None:
                    continue
                # Guard against cycles.
                covered[(permission, permission_manager)] = 1
                grants = permission_manager.grants(prerequisite, object,
                                                   _covered=covered)
                if grants:
                    if grants[0] != (prerequisite, permission_manager):
                        # Add to the explanation a clause that says that
                        # prerequisite was granted by permission_manager.
                        grants = [(prerequisite, permission_manager)] + grants
                    return grants
                del covered[(permission, permission_manager)] # Clean up.
        return None






"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/extent.py $
$Id: extent.py 22651 2003-09-30 02:28:27Z dbinger $

Provides the Extent class.
"""

from __future__ import generators

from time import time, ctime
from ZODB.FileStorage import FileStorage
from dulcinea import base
from dulcinea.database import pack_oid, unpack_oid

class EmptyExtent (Exception):
    """
    Instance attributes: none
    """
    pass

class Extent (base.DulcineaPersistent):
    """
    An Extent maintains a mapping of class name to oid list
    for the objects present in the database.

    Instance attributes:
      mapping : PersistentMapping {class_name:string : {oid:string:boolean}}
      start_oid_scan : long
        The oid to start scanning from. A value of 0L indicates no objects
        have been scanned.
    """

    def __init__ (self):
        self.clear()

    def clear (self):
        self.mapping = base.PersistentMapping()
        self.start_oid_scan = 0L

    def get_database (self):
        """() -> ObjectDatabase"""
        raise NotImplemented, 'subclass must override'

    def objects (self):
        """
        Generate a sequence of all objects.
        """
        for class_name in self.mapping.keys():
            for object in self.all(class_name):
                yield object

    def scan (self):
        """
        Examine the objects that have appeared since the
        last scan, or all objects if no scan has happened
        since the last call of the clear() method.
        Add each object's oid to the appropriate list in
        the values of self.mapping.
        """
        object_db = self.get_database()
        storage = object_db.get_storage()
        connection = object_db.get_connection()
        highest_oid = unpack_oid(storage.new_oid())
        oid = self.start_oid_scan

        while oid < highest_oid:
            packed_oid = pack_oid(oid)
            try:
                object = connection[packed_oid]
            except KeyError:
                pass
            else:
                class_name = (object.__module__ +
                              "." +
                              object.__class__.__name__)
                object._p_deactivate()
                if not self.mapping.has_key(class_name):
                    self.mapping[class_name] = {}
                else:
                    self.mapping[class_name] = self.mapping[class_name]
                self.mapping[class_name][packed_oid] = 1
            oid += 1

        if isinstance(storage, FileStorage):  # exclusive access to DB
            self.start_oid_scan = oid # start next scan here

    def all (self, *class_names):
        """
        Generate a sequence of all objects in the database with any of
        the given class names (strings).  Note that inheritance is not
        used here.
        """
        object_db = self.get_database()
        connection = object_db.get_connection()
        self.scan()

        # Construct a temporary mapping of usable synonyms.
        synonyms = {}
        for full_name in self.mapping.keys():
            class_name = full_name.split('.')[-1]
            if synonyms.has_key(class_name):
                synonyms[class_name] = None # Synonym collision.
            synonyms[class_name] = full_name

        for class_name in class_names:
            oids = (self.mapping.get(class_name) or
                    self.mapping.get(synonyms.get(class_name)))
            if oids is None:
                raise EmptyExtent("no immediate instances of %r here."
                                  % class_name)
            bad_oids = 0
            for oid in oids.keys():
                try:
                    object = connection[oid]
                except KeyError:
                    del oids[oid]
                    bad_oids += 1
                else:
                    dir(object)
                    yield object
            if bad_oids:
                self.mapping._p_changed = 1
                print ("%d instances of %s were deleted by the last pack."
                       % (bad_oids, class_name))

    def count (self, class_name):
        """
        Return the number of objects with the given class name.
        """
        result = 0
        for object in self.all(class_name):
            result += 1
        return result

    def census (self):
        """
        Returns a string containing a formatted census report.
        """
        self.scan()
        pairs = [ (self.count(name), name)
                  for name in self.mapping.keys() ]
        pairs.sort()
        s = '\n    Census (%s):\n\n' % ctime(time())
        sum = 0L
        for pair in pairs:
            sum += pair[0]
            if pair[0] == 0:
                del self.mapping[pair[1]]
            else:
                s += '%11d %s\n' % pair
        s += '    -------\n'
        s += '%11d %s\n\n' % (sum, 'Persistent')
        return s




#!/www/python/bin/python

"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/test/test_country.py $
$Id: test_country.py 20909 2003-03-04 14:35:33Z dbinger $

Test for dulcinea.country.
"""

from types import TupleType
from sancho.unittest import TestScenario, parse_args, run_scenarios
from dulcinea.country import get_country_name, get_country_list

tested_modules = [ "dulcinea.country" ]

class CountryTest (TestScenario):

    def check_country_lookup (self):
        self.test_val("get_country_name('CA')", "Canada")
        self.test_val("get_country_name('AU')", "Australia")
        self.test_val("get_country_name('TV')", "Tuvalu")
        self.test_val("get_country_name('XX')", None)
        self.test_val("get_country_name(None)", None)

    def check_country_list (self):
        list = get_country_list()
        self.test_true("len(list) > 200")
        self.test_true("('FI', 'Finland') in list")
        self.test_true("('BR', 'Brazil') in list")
        self.test_true("0 not in "
                       "[type(x) is TupleType and len(x) == 2 for x in list]")


if __name__ == "__main__":
    (scenarios, options) = parse_args()
    run_scenarios(scenarios, options)



"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/errors.py $
$Id: errors.py 23268 2003-12-04 12:26:43Z dbinger $

Provides DulcineaError, the root class for all Dulcinea exception
classes.  Also provides the Dulcinea specific exception classes.
"""
import quixote.errors

class DulcineaError (Exception):
    """The root of all evil; all exception classes defined in the Dulcinea
    package should subclass from it.

    Could be used to provide common error-handling mechanisms, but for
    now it's just a trivial, empty exception class.

    Instance attributes: none
    """

class UsageError (DulcineaError):
    """Raised when a class is used inappropriately or incorrectly, eg.
    when operations are called out of order or some prerequisite is
    not met.

    Instance attributes: none
    """

class ConflictError (DulcineaError):
    """Raised when a database conflict error is discovered,
    eg. attempting to add a user whose user ID is already in use.

    Instance attributes: none
    """

class NotLoggedInError(quixote.errors.AccessError, DulcineaError):
    """Raised when access is denied because the user should be signed in.

    Instance attributes: none
    """



#!/www/python/bin/python
"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/test/test_history.py $
$Id: test_history.py 23314 2004-01-07 18:24:53Z abenard $
"""

from sancho.unittest import TestScenario, parse_args, run_scenarios
from dulcinea.specified_list import StringList

tested_modules = [ "dulcinea.specified_list" ]

class Test (TestScenario):

    def check_stringlist (self):
        self.test_stmt("s=StringList()")
        self.test_exc("s=StringList([1])", TypeError)
        self.test_stmt("s=StringList(['a'])")
        self.test_stmt("s.append('b')")
        self.test_val("list(s)", ['a', 'b'])
        self.test_stmt("s.insert(1, 'c')")
        self.test_val("list(s)", ['a', 'c', 'b'])
        self.test_stmt("s.insert(0, 'd')")
        self.test_val("list(s)", ['d', 'a', 'c', 'b'])
        self.test_exc("s.insert(0, 3)", TypeError)
        self.test_stmt("s.insert(7, 'e')")
        self.test_val("list(s)", ['d', 'a', 'c', 'b', 'e'])
        self.test_exc("s[0] = 7", TypeError)
        self.test_stmt("s[0] = 'f'")
        self.test_val("list(s)", ['f', 'a', 'c', 'b', 'e'])
        self.test_exc("s[3:4] = ['a', 4]", TypeError)
        self.test_stmt("s[3:4] = ['a', 'b']")
        self.test_val("list(s)", ['f', 'a', 'c', 'a', 'b', 'e'])
        self.test_exc("s.extend(['a', 4])", TypeError)
        self.test_stmt("s.extend(['a', 'b'])")
        self.test_val("list(s)", ['f', 'a', 'c', 'a', 'b', 'e', 'a', 'b'])

if __name__ == "__main__":
    (scenarios, options) = parse_args()
    run_scenarios (scenarios, options)



'''$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/ui/publisher.py $
$Id: publisher.py 22753 2003-10-15 21:26:17Z nascheme $
'''

import os
import time
import random

from ZODB.POSException import ConflictError
from quixote.publish import SessionPublisher
from quixote.http_response import HTTPResponse
from dulcinea import local
from dulcinea import site_util
from dulcinea.database import get_transaction

class DulcineaPublisher (SessionPublisher):

    # times to retry a request that failed due to a conflict error
    CONFLICT_RETRIES = 3

    def __init__(self):
        config = site_util.get_config()
        root_namespace = config.get(os.environ['SITE'], 'root-namespace')
        SessionPublisher.__init__(self, root_namespace,
                                  session_mgr=local.get_session_manager())

    def start_request (self, request):
        local.get_connection().sync()
        SessionPublisher.start_request(self, request)

    def try_publish (self, request, path):
        """Try to process a single request.  Use the base class method to the
        do actual the work.  If a conflict error occurs retry a number of
        times.
        """
        for i in range(self.CONFLICT_RETRIES + 1):
            try:
                return SessionPublisher.try_publish(self, request, path)
            except ConflictError, exc:
                self.debug("ConflictError (%s): retrying request" % `exc.oid`)
                get_transaction().abort()
                request.response = HTTPResponse() # reset response object
                time.sleep(random.random() * (i+1))
        else:
            raise RuntimeError("too many conflict errors")



#!/www/python/bin/python
"""
$URL$
$Id$
"""

from sancho.unittest import TestScenario, parse_args, run_scenarios
from dulcinea.links import LinkItem, LinkFolder, sort_reverse_by_date

tested_modules = ["dulcinea.links"]


class LinksTest (TestScenario):


    def check_link(self):
        self.test_stmt("self.link = LinkItem()")
        self.test_exc("self.link.set_url('bad url')", AssertionError)
        self.test_stmt("self.link.set_url('http://www.mems-exchange.org')")
        self.test_val("self.link.get_url()", 'http://www.mems-exchange.org')
        self.test_stmt("self.link2 = LinkItem()")
        self.test_stmt("[self.link, self.link2].sort(sort_reverse_by_date)")

    def check_link_folder(self):
        self.test_stmt("link_db = LinkFolder()")


if __name__ == "__main__":
    (scenarios, options) = parse_args()
    options.show_coverage = 1
    #options.show_coverage_lines = 1
    run_scenarios(scenarios, options)



#!/www/python/bin/python
"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/test/test_address.py $
$Id: test_address.py 23372 2004-01-23 20:45:08Z dbinger $
"""

from sancho.unittest import TestScenario, parse_args, run_scenarios
from dulcinea.address import Address, is_valid_phone_number

tested_modules = ["dulcinea.address"]

class AddressTest (TestScenario):

    test_cases = ['init', 'complete', 'valid']

    def check_init (self):
        self.test_stmt('Address()')
        self.test_stmt('a=Address(street1="s1", street2="s2")')
        self.test_val('a.street1', 's1')
        self.test_val('a.street2', 's2')
        self.test_exc('Address(bogus="goofy")', TypeError)

    def check_complete(self):
        addr = Address()
        addr.contact_name = "Bogus Contact"
        addr.contact_phone_number = "888 555 1212"
        self.test_true("addr.has_valid_phone_number()")

        # Check a US address
        addr.street1 = '1320 St.'
        addr.city = 'Arlington'
        addr.state = 'VA' ; addr.country_code = 'US'
        addr.zip = '22201'
        self.test_seq('addr.check_complete()', [])

        # Test an empty contact phone number
        addr.contact_phone_number = ''
        self.test_seq('addr.check_complete()', [('contact_phone_number',
                                                 'missing')])
        addr.contact_phone_number = "888 555 1212"

        # Test the checks on the state field
        addr.state = ''
        self.test_seq('addr.check_complete()',
                      [('state', 'missing')])

        addr.state = 'VAR'
        self.test_seq('addr.check_complete()',
                      [('state', 'bad')])

        addr.state = 'VB'
        self.test_seq('addr.check_complete()',
                      [('state', 'bad')])

        addr.state = 'VA'
        addr.zip = '29999-'
        self.test_seq('addr.check_complete()',
                      [('zip', 'bad')])

        addr.state = 'XX'
        addr.zip = None
        self.test_seq('addr.check_complete()',
                      [('state', 'bad'),
                       ('zip', 'missing')])

        # Will a Canadian province work?
        addr.country_code = 'CA' ; addr.state = 'QC'
        self.test_seq('addr.check_complete()', [])


    def check_valid (self):
        addr = Address()
        self.test_true("addr.is_empty()")
        self.test_true("len(addr.check_complete()) > 0")
        self.test_false("addr.is_valid()")

        addr.street1 = "123 Main St."
        self.test_false("addr.is_empty()")
        self.test_true("len(addr.check_complete()) > 0")
        self.test_false("addr.is_valid()")

        addr.city = "Paris"
        addr.country_code = "FR"
        self.test_seq("addr.check_complete()", [('contact_name',
                                                 'missing'),
                                                ('contact_phone_number',
                                                 'missing')])
        self.test_false("addr.is_empty()")
        self.test_false("addr.is_valid()")
        addr.contact_name = 'foo'
        addr.contact_phone_number = '1112223333'
        self.test_true("addr.is_valid()")

    def check_friendly (self):
        "check that we can get human-friendly attribute names"
        addr = Address()
        self.test_val("addr.get_friendly_attr('contact_phone_number')",
                      "phone number")
        self.test_val("addr.get_friendly_attr('city')", "city")
        self.test_val("addr.get_friendly_attr('street1')", "street address")
        self.test_val("addr.get_friendly_attr('zip')", "postal/ZIP code")
        self.test_exc("addr.get_friendly_attr('foo')", AttributeError)

    def check_phone (self):
        "test is_valid_phone_number()"

        # Let's see, how many ways are there to write a North
        # American phone number...
        self.test_true("is_valid_phone_number('514-482-7346')")
        self.test_true("is_valid_phone_number('1-514-482-7346')")
        self.test_true("is_valid_phone_number('+1-514-482-7346')")
        self.test_true("is_valid_phone_number('5144827346')")
        self.test_true("is_valid_phone_number('514 482 7346')")
        self.test_true("is_valid_phone_number('(514) 482-7346')")
        self.test_true("is_valid_phone_number('1 (514) 482-7346')")
        self.test_true("is_valid_phone_number('+1 (514) 482-7346')")

        # Sorry, gotta have the area code.
        self.test_false("is_valid_phone_number('482 7346')")
        self.test_false("is_valid_phone_number('482-7346')")

        # Lots of offices have extensions, and there are lots of ways to
        # spell them.
        self.test_true("is_valid_phone_number('703-620-8990 x123')")
        self.test_true("is_valid_phone_number('703-620-8990 x 123')")
        self.test_true("is_valid_phone_number('703-620-8990 ext123')")
        self.test_true("is_valid_phone_number('703-620-8990 ext. 123')")
        self.test_true("is_valid_phone_number('703-620-8990 X123')")
        self.test_true("is_valid_phone_number('703-620-8990 X 123')")
        self.test_true("is_valid_phone_number('703-620-8990 Ext123')")
        self.test_true("is_valid_phone_number('703-620-8990 Ext. 123')")

        # Try some international numbers.
        self.test_true("is_valid_phone_number('+44 (0)20 8681 9603')")
        self.test_true("is_valid_phone_number('+33 01 47 42 20 34')")
        self.test_true("is_valid_phone_number('+33 03 83 27 56 52')")

if __name__ == "__main__":
    (scenarios, options) = parse_args()
    run_scenarios (scenarios, options)




"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/base.py $
$Id: base.py 23388 2004-02-03 00:14:30Z rmasse $

Provides universal base classes which all classes in the Dulcinea
library should subclass.  Currently there are two such
universal base classes:
    DulcineaPersistent    for classes whose instances should always be
                          first-class, persistent database objects
    DulcineaBase          for classes whose instances may be transient,
                          but can still be persisted if they happen to
                          be attached to another persistent object

"""

import new
from types import ClassType
from ZODB import DB, utils
from ExtensionClass import ExtensionClass
from Persistence import Persistent


# If you're wondering which of DulcineaBase and DulcineaPersistent to use
# for some class, then use DulcineaPersistent: that will make instances of
# that class "first-class" database objects, subject to all of ZODB's
# machinery.  Usually this is a good thing, but keep in mind that deriving
# from DulcineaPersistent means your class is an ExtensionClass, which has
# a number of ramifications:
#   * the '__radd__()' (and, presumably, '__rmul__()', '__rsub__()',
#     '__rdiv__()') protocol doesn't work: this is a bug in
#     ExtensionClass.
#   * the '__cmp__()' protocol may not work as expected (unsure on
#     the details here)
#
# In short, be leery of ExtensionClass (and thus DulcineaPersistent) for
# any class that needs to do operator overloading: this stuff is rather
# flaky for ExtensionClasses.  Conversely, only do operator overloading if
# it's really necessary and a big win, since it means that objects of
# classes with operator overloads won't be able to be first-class database
# objects, at least until the '__radd__()' bug in ExtensionClass is fixed.


class DulcineaBase:
    """The standard base class for classes whose instances do not need
    to be first-class database objects.  Instances of DulcineaBase can still
    be stored in the database if they happen to be attached to
    other persistent objects, but they don't really participate in
    persistence -- they just come along for the ride.

    Instance attributes: none

    """

    # Our standard __repr__(): like Python's, except it also puts the
    # str() of the object in the repr() and uses bare class name
    # (no full module path).
    def __repr__ (self):
        if hasattr(self, '__str__'):
            strvalue = ": " + str(self)
        else:
            strvalue = ""

        return ("<%s at %08x%s>" % (self.__class__.__name__,
                                   id(self), strvalue))

    def copy (self):
        klass = self.__class__
        if type(klass) is ExtensionClass:
            newself = klass.__basicnew__()
        elif type(klass) is ClassType:
            newself = new.instance(self.__class__, {})
        elif isinstance(klass, object):
            newself = klass.__new__(klass)
        else:
            raise RuntimeError, "unknown type of class object: %s" % `klass`

        for attr in self.__dict__.keys():
            if attr[0:3] not in ("_p_", "_v_"):
                val = getattr(self, attr)
                setattr(newself, attr, val)

        return newself

    def __copy__ (self):
        return self.copy()


class DulcineaPersistent (DulcineaBase, Persistent):
    """The standard base class for classes whose instances should be
    first-class database objects that participate fully in the
    persistence mechanism.  Instances of DulcineaPersistent still have to be
    referenced by an existing persistent object, i.e. they must
    ultimately connect back to one of the database root objects.

    Instance attributes: none
    """

    # used below by __repr__ to determine if __str__ has been overridden
    _default__str__ = getattr(Persistent, "__str__", None)

    def __repr__ (self):
        str_method = getattr(self.__class__, '__str__', None)
        if str_method is not self.__class__._default__str__:
            # subclass overrode __str__ method, it's safe to call str()
            strvalue = ": " + str(self)
        else:
            # default __str__ falls back to __repr__ so don't call it
            strvalue = ""
        if self._p_oid:                 # brand-new objects don't have OID yet
            oid = "%016x" % utils.U64(self._p_oid)
        else:
            oid = "%08x" % id(self)
        return "<%s at %s%s>" % (self.__class__.__name__, oid, strvalue)

    def __hash__ (self):
        # 'object' provides a default hash method but ExtensionClass does not
        return id(self)


from Persistence import PersistentMapping
def _persistent_map_copy(self):
    return self.__class__(self.data)
PersistentMapping.__copy__ = _persistent_map_copy
PersistentMapping.copy = _persistent_map_copy

# XXX ZODB4 drops support for old _container attribute.  We should
# re-commit all PersistentMapping objects and remove this code.
def _persistent_map_setstate(self, dict):
    self.__dict__.update(dict)
    if hasattr(self, '_container'):
        self.data = self._container
        del self._container
PersistentMapping.__setstate__ = _persistent_map_setstate









#!/www/python/bin/python

"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/test/test_child_process.py $
$Id: test_child_process.py 20909 2003-03-04 14:35:33Z dbinger $

Test for dulcinea.child_process.
"""

from sancho.unittest import TestScenario, parse_args, run_scenarios
from dulcinea import child_process

tested_modules = ["dulcinea.child_process" ]

class ChildProcessTest (TestScenario):

    def check_cat (self):
        p = child_process.execute(('cat',))
        self.test_stmt('p.write("hello world\\n")')
        self.test_stmt('p.stdin.close()')
        self.test_val('p.readline()', 'hello world\n')
        self.test_val('p.close()', None)

    def check_ls (self):
        p = child_process.execute(("ls", "/"))
        self.test_val('p.readline()', 'bin\n')
        self.test_val('p.readline()', 'boot\n')
        self.test_val('p.close()', None)

    def check_error (self):
        p = child_process.execute(("false",))
        self.test_val('p.close()', 256)


if __name__ == "__main__":
    (scenarios, options) = parse_args()
    run_scenarios(scenarios, options)



#!/www/python/bin/python

"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/bin/start-zeo.py $
$Id: start-zeo.py 21927 2003-06-26 15:22:47Z nascheme $

Script invoked by the site command to start zeod.
"""

import sys, os

from dulcinea import site_util
site_util.ensure_uid_gid_not_root()
config = site_util.get_config()
site = sys.argv[1]
pid_file_name = site_util.get_pid_file_name ('zeo', site)
ip, port = site_util.parse_address(config.get(site, 'zeo-address'))
assert site_util.is_local(config.get(site, 'zeo-address'))
db = os.path.join(config.get(site, 'var-directory'), '%s.fs' % site)

site_log_dir = os.path.join(config.get(site, 'log-directory'), site)
zeo_log = os.path.join(site_log_dir, 'zeo.log')

zeod = config.defaults().get('zeod')
os.execve(zeod, (zeod, '-p', str(port), '-P', pid_file_name, db),
          {'STUPID_LOG_FILE': zeo_log, 'STUPID_LOG_SEVERITY': '100'}
         )





#!/www/python/bin/python
"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/test/test_stored_file.py $
$Id: test_stored_file.py 23441 2004-02-05 22:00:46Z dbinger $

Test script for the StoredFile and related classes.
"""

import os, shutil
from mx.DateTime import now
from sancho.unittest import TestScenario, parse_args, run_scenarios
from dulcinea.stored_file import StoredFile, path_check, FileDatabase
from dulcinea.user import DulcineaUser

tested_modules = ["dulcinea.stored_file"]

TEST_ROOT_DIR = '/tmp/storedfiletest'
TEST_CONTENT = "Contents of a file go here."

class StoredFileTest (TestScenario):

    def setup (self):
        if os.path.exists(TEST_ROOT_DIR):
            shutil.rmtree(TEST_ROOT_DIR)
        os.mkdir(TEST_ROOT_DIR)
        self.user = DulcineaUser('a')
        self.content_filename = os.path.join(TEST_ROOT_DIR, 'test_content')
        content = open(self.content_filename, 'w')
        content.write(TEST_CONTENT)
        content.close()

    def shutdown (self):
        shutil.rmtree(TEST_ROOT_DIR)
        del self.content_filename

    def check_file (self):
        self.test_stmt("self.file_db = FileDatabase(root_directory=%r)" %
                       TEST_ROOT_DIR)
        self.test_exc(
            'sf = self.file_db.new_file(source_path=self.content_filename,'
            'allowed_mime_types=[1])', TypeError)
        self.test_stmt(
            'sf = self.file_db.new_file(source_path=self.content_filename)')

        self.test_stmt('self.f = sf.open("rt")')
        self.test_val('self.f.read()', TEST_CONTENT)
        self.f.close()

        self.test_stmt('self.f = sf.open("wb")')
        self.content = 'New binary content goes here...\n'
        self.test_stmt("self.f.write(self.content)")
        self.f.close()
        self.test_val('sf.get_size()', len(self.content))
        self.test_stmt('self.f = sf.open("r")')
        self.test_val('self.f.read()', self.content)
        self.f.close()
        self.test_exc('sf.set_mime_type("textplain")', ValueError)
        self.test_stmt('sf.set_mime_type("text/plain")')
        self.test_val('sf.guess_mime_type()', 'text/plain')
        self.test_stmt('sf.MIME_TYPES = []')
        self.test_val('sf.guess_mime_type()', 'application/octet-stream')
        self.test_val('sf.get_mime_type()', 'text/plain')
        self.test_stmt('self.file_db.get_file(sf.path)==sf')
        self.test_stmt('sf.set_filename("foo")')
        self.test_val('sf.get_filename()', 'foo')
        self.test_stmt('sf.set_description("boo")')
        self.test_val('sf.get_description()', 'boo')
        u = DulcineaUser()
        self.test_stmt('sf.set_owner(u)')
        self.test_val('sf.get_owner()', u)
        self.test_true('sf.has_manage_access(u)')
        adate = now()
        self.test_stmt('sf.set_date(adate)')
        self.test_val('sf.get_date()', adate)

        self.test_stmt('sf.chgrp("mxweb")')
        self.test_stmt('sf.chgrp(4130)')
        self.test_stmt('self.file_db.remove_file(sf, unlink=True)')
        self.test_false('os.path.exists(sf.get_full_path())')
        self.test_val('sf.get_size()', None)
        self.test_stmt('repr(sf)')
        self.test_stmt('sf.get_path()')


    def check_path_check (self):
        self.test_val('path_check("Abc")', 'abc')
        self.test_val('path_check("abc/../../../etc/passwd")', None)
        self.test_val('path_check("/special/characters/-_@.cif")',
                      "/special/characters/-_@.cif")
        self.test_val('path_check("/illegal/characters\001/")', None)


if __name__ == "__main__":
    (scenarios, options) = parse_args()
    run_scenarios(scenarios, options)



#!/www/python/bin/python

"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/test/test_user.py $
$Id: test_user.py 23216 2003-11-21 21:00:24Z dbinger $

Test for dulcinea.user.
"""

from sancho.unittest import TestScenario, parse_args, run_scenarios
from dulcinea.user import DulcineaUser as User

tested_modules = [ "dulcinea.user" ]

class DulcineaUserTest (TestScenario):

    def check_init (self):
        "Check constructor"
        self.test_exc("User('!@#$%^')", ValueError)
        self.test_stmt("User('amk')")
        self.test_stmt("u = User() ; u.set_id('foo')")
        self.test_exc("User('foo').set_id('foo')", AssertionError)
        self.test_exc("u = User() ; u.set_id('!foo')", ValueError)

        u = User()
        self.test_val("u.id", None)
        self.test_stmt("u.set_id('foo')")
        self.test_val("u.id", "foo")
        self.test_exc("u.set_id('bar')", AssertionError)

    def check_password (self):
        "Test password-related methods"
        user = User('foo')
        self.test_stmt('user.set_password("biteme")')
        self.test_true('user.valid_password("biteme")')
        self.test_false('user.valid_password("dontbiteme")')
        self.test_stmt('password1 = user.generate_password()')
        self.test_true('user.valid_password(password1)')
        self.test_stmt('password2 = user.generate_password()')
        self.test_true('user.valid_password(password2)')
        self.test_true('password1 != password2')


if __name__ == "__main__":
    (scenarios, options) = parse_args()
    run_scenarios(scenarios, options)



"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/stored_file.py $
$Id: stored_file.py 23441 2004-02-05 22:00:46Z dbinger $

Store a file in the filesystem.
"""

import os, string, grp
import shutil
import time
from mx import DateTime
from dulcinea import site_util
from dulcinea.spec import any, mapping
from dulcinea.typeutils import typecheck
from dulcinea.base import DulcineaPersistent, PersistentMapping
from dulcinea.user import DulcineaUser

ALLOWABLE_SPECIAL_CHARS = "-_.@"
_last_name = None


def path_check(path):
    """(path:string) -> string | None

    Return a lower-case and normalized version of the 'path' after
    checking for allowable characters.
    Return None if path has '..' or any special characters
    other than what's in string.lowercase, string.digits, and
    the ALLOWABLE_SPECIAL_CHARS string
    """
    if '..' in path:
        return None
    path = path.lower()
    for letter in path:
        if (letter != '/' and letter not in string.lowercase and
            letter not in string.digits and
            letter not in ALLOWABLE_SPECIAL_CHARS):
            return None
    return path


class StoredFile (DulcineaPersistent):
    """Class for reading from or writing to a stored file

    A StoredFile is a wrapper around a file in a place in the
    filesystem known as the 'file-store'.

    StoredFiles can be instantiated with a fresh source file to move a new
    file into the file-store, or instantiated by referencing an existing
    file in the file-store.

    Instance attributes:
      path : string
        Path to the physical file containing the stored file's data.
      full_path : string
        The full path with nothing omitted.
      filename : string
        Filename to use when downloading this file
      mime_type : string
        MIME type of the contents of this file.
      description : string
        Description of this file.
      owner : DulcineaUser
        The file's owner
      date : DateTime
        timestamp
    """

    MIME_TYPES = [("image/gif", "GIF image"),
                  ("image/jpeg", "JPEG image"),
                  ("image/png", "PNG image"),
                  ("image/tiff", "TIFF image"),
                  ("image/x-ms-bmp", "BMP image"),
                  ("text/plain", "plain text"),
                  ("text/html", "HTML text"),
                  ("text/rtf", "richtext document"),
                  ("application/pdf", "PDF document"),
                  ("application/msword", "Word document"),
                  ("application/vnd.ms-powerpoint", "PowerPoint document"),
                  ("application/postscript", "PostScript file"),
                  ("application/octet-stream", "Binary data")]


    path_is = str
    full_path_is = str
    filename_is = str
    mime_type_is = any(*[pair[0] for pair in MIME_TYPES])
    description_is = any(str, None)
    owner_is = DulcineaUser
    date_is = DateTime.DateTimeType

    def __init__(self, path, root_dir, source_path=None):
        """(path: str, root_dir : string, source_path : string)

        If 'source_path' exists, copy the file specified to the file-store
        location specified in 'path'.  'path' specifies a relative path, so
        the full path is root_dir + path.

        If 'path' contains directories, traverse into those directories
        creating them as needed. 'source_path' must be readable if specified.
        """
        self.description = None
        self.owner = None
        self.date = DateTime.now()

        # Check path for safety (no '..' components, that sort of thing)
        path = os.path.normpath(path)
        self.path = path_check(path)
        assert self.path, 'Bad name: %r' % path
        self.filename = os.path.basename(self.path)

        self.full_path = os.path.join(root_dir, self.path)
        assert not os.path.exists(self.full_path), (
            'path %r already exists' % self.full_path)

        if source_path:
            # Copy the file specified in source_path into the file-store
            f = open(source_path, mode='rb')
            f2 = self.open(mode='wb')
            shutil.copyfileobj(f, f2)
            f.close()
            f2.close()

        self.mime_type = self.guess_mime_type()

    def __repr__(self):
        return '<%s: path=%r>' % (self.__class__.__name__, self.path)

    def get_path(self):
        return self.path

    def guess_mime_type(self):
        p = os.popen("file -b --mime '%s' 2>/dev/null" % self.get_full_path())
        mime_type = p.read().strip()
        if ';' in mime_type:
            mime_type = mime_type.split(';')[0]
        status = p.close() # 'file' seems to always return error status 0!
        # only allow MIME types we know about
        for (type, description) in self.MIME_TYPES:
            if type == mime_type:
                break
        else:
            mime_type = "application/octet-stream"
        return mime_type

    def open(self, mode='rb'):
        """(mode:string) -> file
        Returns a Python file object opened for reading or writing,
        depending on the 'mode' parameter.  This can be used to read or
        update the file's content.
        """
        full_path = self.get_full_path()
        if 'w' in mode:
            dir = os.path.dirname(full_path)
            if not os.path.exists(dir):
                os.makedirs(dir, mode=0775)
        file = open(full_path, mode=mode)
        return file

    def chgrp(self, grp_id):
        """(grp_id : str | int)
        Change the group ownership of this file to the specified ID.
        """
        if isinstance(grp_id, str):
            grnam, grpass, gid, grmembers = grp.getgrnam(grp_id)
            grp_id = gid
        os.chown(self.get_full_path(), -1, grp_id)

    def set_mime_type(self, mime_type):
        """(mime_type:string)
        Sets the MIME type for the file.
        """
        # Check there's a single '/' in the MIME type
        if mime_type.count('/') != 1:
            raise ValueError, "Invalid MIME type %r" % mime_type
        self.mime_type = mime_type

    def get_mime_type(self):
        return self.mime_type

    def set_filename(self, filename):
        self.filename = filename

    def get_filename(self):
        return self.filename

    def set_description(self, description):
        self.description = description

    def get_description(self):
        return self.description

    def set_date(self, date):
        typecheck(date, DateTime.DateTimeType)
        self.date = date

    def get_date(self):
        return self.date

    def set_owner(self, owner):
        typecheck(owner, DulcineaUser)
        self.owner = owner

    def get_owner(self):
        return self.owner

    def get_size(self):
        """() -> int
        Return the size of the file, measured in bytes, or None if
        the file doesn't exist.
        """
        path = self.get_full_path()
        if not os.path.exists(path):
            return None
        stats = os.stat(path)
        return stats.st_size

    def get_full_path(self):
        """() -> string
        """
        return self.full_path

    def has_manage_access(self, user):
        return (user is self.owner or user.is_admin())


class FileDatabase (DulcineaPersistent):
    """
    Instance attributes:
      root_directory: string
      files : PersistentMapping { string : StoredFile }
      """

    root_directory_is = str
    files = mapping({str: StoredFile}, PersistentMapping)

    def __init__(self, root_directory=None):
        if root_directory is None:
            site = os.environ['SITE']
            config = site_util.get_config()
            root_directory = config.get(site, 'file-store',
                                        fallback='/tmp')
        self.root_directory = root_directory
        self.files = PersistentMapping()

    def get_file(self, path):
        return self.files[path]

    def new_file(self, source_path=None, allowed_mime_types=None):
        """(source_path : string, allowed_mime_types : [string]) -> StoredFile
        Returns a new StoredFile object.  The path for the file is generated
        using the current time, and will be unique (Note: for this process).
        """
        global _last_name
        while 1:
            now = str(long(10*time.time()))
            if now != _last_name:
                break
            time.sleep(0.1)
        _last_name = now
        path = '%s/%s/%s/' % (now[-1], now[-3:-1], now)
        sf = StoredFile(path, self.root_directory, source_path=source_path)
        if allowed_mime_types and sf.get_mime_type() not in allowed_mime_types:
            raise TypeError, "MIME type '%s' not allowed, allowed types: %s" % (
                sf.get_mime_type(), allowed_mime_types)
        self.add_file(sf)
        return sf

    def add_file(self, stored_file):
        self.files[stored_file.path] = stored_file

    def remove_file(self, stored_file, unlink=False):
        del self.files[stored_file.path]
        if unlink:
            os.unlink(stored_file.get_full_path())



#!/www/python/bin/python

"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/bin/start-apache.py $
$Id: start-apache.py 22952 2003-11-03 00:14:26Z dbinger $

Script invoked by site command to start apache.
"""

import os
from dulcinea import site_util

config = site_util.get_config()
sites_directory = config.defaults().get('sites-directory')

var_directory = config.defaults().get('var-directory')
httpd_conf_name = os.path.join(var_directory, 'generated-httpd.conf')

httpd_conf = open(httpd_conf_name, 'w')

def w (*s):
    httpd_conf.write(' '.join(map(str, s)) + '\n')

def include_site_conf (include_file, site):
    file_name = os.path.join(sites_directory,
                             site,
                             'conf',
                             include_file)
    if os.path.exists(file_name):
        w('Include', file_name)

def include_global_conf (conf_file_name):
    full_name = os.path.join(config.defaults().get('conf-directory'),
                             'apache',
                             conf_file_name)
    if os.path.exists(full_name):
        w('Include', full_name)

def common (site):
    servername = config.get(site, 'servername')
    if servername is None:
        servername = site_util.get_hostname()
    w('ServerName %s' % servername)
    scgi_address = config.get(site, 'scgi-address')
    if scgi_address:
        exports = list(site_util.get_root_exports(site))
        exports.sort()
        if '_q_index' in exports: # dynamic root
            exports.append('$')
        w('<LocationMatch "^/(%s)($|/.*)">' % '|'.join(exports))
        w('    SCGIServer %s %s' % site_util.parse_address(scgi_address))
        w('    SCGIHandler On')
        w('</LocationMatch>')
    include_site_conf('apache-common.conf', site)

w('# Generated. Edit site.conf and/or start_apache.py to change.')
w('ServerAdmin', site_util.get_administrator_address()[0])
w('Port', 80)
w('User', config.defaults().get('daemon-uid'))
w('PidFile', site_util.get_pid_file_name('apache'))

is_live = site_util.any_live_sites()

ports = {}
for site in site_util.list_sites():
    for option in ('http-address', 'https-address'):
        if config.has_option(site, option):
            ip, port = site_util.parse_address(config.get(site, option))
            ports[port]=1
for port in ports:
    w('Listen', port)


if not is_live:
    # Directives if not live for any site.
    w('CoreDumpDirectory', config.defaults().get('var-directory'))
    w('LogLevel debug')

include_global_conf('apache-global.conf')

for site in site_util.list_sites():
    if not config.has_option(site, 'http-address'):
        continue
    if (is_live and not site_util.is_live(site)):
        # In live mode, skip sections that aren't live 
        continue

    docroot = '/www/sites/%s/docroot' % site
    if not os.path.exists(docroot):
        continue
    host_iface = config.get(site, 'http-address')
    host_ssl_iface = config.get(site, 'https-address')

    w('\n<VirtualHost %s>' % host_iface, '# %s http' % site)
    w('DocumentRoot ' + docroot)
    common(site)
    include_site_conf('apache-http.conf', site)
    w('</VirtualHost>')
    if host_ssl_iface:
        w('\n<VirtualHost %s>' % host_ssl_iface,
          '# %s https' % site)
        w('DocumentRoot ' + docroot)
        common(site)
        w('SSLEngine on')
        w('SSLOptions +StdEnvVars')
        ssl_cert = config.get(site, 'ssl-certificate')
        ssl_cert_key = ssl_cert.replace('.crt', '.key')
        w('SSLCertificateFile %s' % ssl_cert)
        w('SSLCertificateKeyFile %s' % ssl_cert_key)
        w('</VirtualHost>')
    if site_util.is_live(site):
        include_site_conf('apache-live.conf', site)

httpd_conf.close()

httpd = config.defaults().get('httpd')
os.execv(httpd, [httpd, '-f', httpd_conf_name])




#!/www/python/bin/python
"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/bin/make_tab_edge_images.py $
$Id: make_tab_edge_images.py 23261 2003-12-03 16:40:00Z dbinger $

Supports generating tab-edge png files in any color.

Example:
make_tab_edge_images.py unselected 9 40 102 56 88 160
make_tab_edge_images.py selected 183 200 226 56 88 160

Note: tab.ptl currently uses gifs made from these by
using pngcrush, opening the tiny pngs in xv, changing to 8-bit mode,
and then saving (normal size) as gif files.
The biggest of the resulting gif files is 232 bytes.
"""

import sys
from PIL import Image
import cStringIO
_MASK = Image.open(cStringIO.StringIO('\x89PNG\r\n\x1a\n\x00\x00\x00\rIHDR\x00\x00\x00)\x00\x00\x00)\x08\x06\x00\x00\x00\xa8`\x00\xf6\x00\x00\x00\x04gAMA\x00\x00\xb1\x8f\x0b\xfca\x05\x00\x00\x01\x0eIDATx\x9c\xcd\x98\xc9\x11\x830\x10\x04\xdb.\x7f\x1d\x82\xb3PF\xc4C,$B\x14\x84\xc0\x97\x07~\xd8\xf8\xc0\xe6\x90\x98\x95v\xaa\xf6\t\xd5\xcc\xee\xe8\x00|\xab\x01\xfa\xd2\x10K\xaa\x80\x01\x18\x9f\xe5J\x01\xe8x\xc3\xb9\x83\xac\xf9\x85s\x05\xf9\xcf=7\x90\x81\xef\xd9s\x07\xb9\x17\xb0\x18\xe4\xda\xfc\xb9\x80\x8c\x05\xcc\x0e\x19\xd3\xe2"\x90\xa9\x80\xe3)#\xe4\x00\\R\x1e<\x8bA\x96\xd4\x91\x08\x98K)A\xc9:\x93\xc9s\x98\x13rk\xbb+\x0eY\x89\x00M\xd3\x9d\x9c\xe6\xb9\xac\xd2\xdd L\xb3\x95\x932\x17\xc1\xc6\xc9\x1a\xf1\x9ah\xe1d\x0f\\\x95/T;Y!\x06\xb4P\x8bh\xd9\x99\x95L\xc1\x08P\ni\xe5\xa2\x0c\xd2\xd2E\xd9\x8e#O\xf4\xa7\x14\xe9\xaeq\x9eh\xd5Q\xcc\xb4\xdd\xa6m\x9et\xa4\xdd-\xce\xdb\xac\xb8\x12\x98.A\xb9\x01\xa3!K\x00FA\x96\x02\xdc\x9d\xee\x0e\xb8\xc5|\x91R[\xe9\x9e\xfe]\x17\x03\\S\xc0\xf6\xc0ph&\xbd\xc1\xbd \x03\x8fP\xa8.\xf2\xf2\xba\x03Y\xb3\x071\x1b\xb2\xb9\xaa\x00\x00\x00\x00IEND\xaeB`\x82'))

def get_tab_image(foreground, background, flip):
    fg_image = Image.new('RGB', _MASK.size, foreground)
    bg_image = Image.new('RGB', _MASK.size, background)
    image = Image.composite(fg_image, bg_image, _MASK)
    if flip:
        image = image.transpose(Image.FLIP_LEFT_RIGHT)
    output = cStringIO.StringIO()
    image.save(output, "PNG")
    return output.getvalue()

def get_left_tab_image(foreground, background):
    return get_tab_image(foreground, background, False)

def get_right_tab_image(foreground, background):
    return get_tab_image(foreground, background, True)

if __name__ == '__main__':
    try:
        base = sys.argv[1]
        numbers = map(int, sys.argv[2:])
        foreground = tuple(numbers[:3])
        background = tuple(numbers[3:6])
    except:
        print """
Usage:
  make_tab_edge_images.py <basename> <fgred> <fggreen> <fgblue> <bgred> <bggreen> <bgblue>
  (writes to <basename>_left.png and <basename>_right.png)
"""
    open(base+'_left.png', 'wb').write(
        get_left_tab_image(foreground, background))
    open(base+'_right.png', 'wb').write(
        get_right_tab_image(foreground, background))
        
        




#!/www/python/bin/python
'''$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/bin/start-scgi.py $
$Id: start-scgi.py 23273 2003-12-05 19:45:15Z nascheme $

Script invoked by site command to start scgi.
'''

import os, sys
import ZODB, ZEO
import pwd
import socket
from scgi.quixote_handler import QuixoteHandler, scgi_server, debug
from dulcinea import local
from dulcinea.database import get_transaction
from dulcinea import site_util
from dulcinea.ui.publisher import DulcineaPublisher

site = os.environ['SITE'] = sys.argv[1]

site_util.ensure_uid_gid_not_root()
config = site_util.get_config()
pid_file_name = site_util.get_pid_file_name('scgi', site)
ip, port = site_util.parse_address(config.get(site, 'scgi-address'))
site_log_dir = os.path.join(config.get(site, 'log-directory'), site)
error_log = os.path.join(site_log_dir, 'error.log')
assert site_util.is_local(config.get(site, 'scgi-address'))

def create_site_publisher(site):
    local.open_database()
    publisher = DulcineaPublisher()
    # set config options
    publisher.config.form_tokens = 1
    publisher.config.display_exceptions = 0
    publisher.config.secure_errors = 1
    publisher.config.session_cookie_path = '/'
    site_log_dir = os.path.join(config.get(site, 'log-directory'), site)
    publisher.config.access_log = os.path.join(site_log_dir, 'access.log')
    publisher.config.error_log = error_log
    publisher.config.compress_pages = 1
    administrator = site_util.get_administrator_address()
    if site_util.is_live(site):
        publisher.config.error_email = administrator[0]
        publisher.config.mail_from = administrator
    elif site_util.is_staging(site):
        publisher.config.error_email = administrator[0]
        publisher.config.mail_from = administrator
        publisher.config.mail_debug_addr = administrator[0]
    else: # devel
        publisher.config.display_exceptions = 1
        publisher.config.secure_errors = 0
        publisher.config.fix_trailing_slash = 0
        user = pwd.getpwuid(os.geteuid())[0] # Owner of the SCGI process
        user_email = user + '@' + socket.getfqdn()
        publisher.config.mail_debug_addr = user_email
        publisher.config.mail_from = user_email
    # Adjust any values that are set in the site configuration.
    for config_var in publisher.config.config_vars:
        if config.has_option(site, config_var):
            setattr(publisher.config, config_var,
                    config.get(site, config_var))
    publisher.setup_logs()

    # We have to commit the current transaction here, just in case there wasn't
    # an existing session manager object in the ZODB.  In the case,
    # get_session_manager() created a new session manager and added it to the
    # ZODB root, but then start_request() will call sync(), losing the
    # modification to the ZODB root.  The net result is that, without the
    # following commit, the session manager is never written to disk and
    # sessions are always immediately thrown away.
    get_transaction().commit()

    return publisher


class Handler (QuixoteHandler):
    prefix = ""

    def __init__(self, *args, **kwargs):
        scgi_server.SCGIHandler.__init__(self, *args, **kwargs)
        self.publisher = create_site_publisher(site)
        debug("SCGI server for %s starting up (pid %d)" % (site, os.getpid()))


site_util.log_to(error_log)

if site_util.is_live(site) or site_util.is_staging(site):
    max_children = 5
else:
    max_children = 1 # faster for development

pid = os.fork()
if pid == 0:
    pid = os.getpid()
    pidfile = open(pid_file_name, 'w')
    pidfile.write(str(pid))
    pidfile.close()
    try:
        scgi_server.SCGIServer(Handler, port=port,
                               max_children=max_children).serve()
    finally:
        # grandchildren get here too, don't let them unlink the pid
        if pid == os.getpid():
            try:
                os.unlink(pid_file_name)
            except OSError:
                pass




#!/www/python/bin/python
"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/test/test_survey.py $
$Id: test_survey.py 23466 2004-02-13 19:12:31Z dbinger $
"""

from sancho.unittest import TestScenario, parse_args, run_scenarios
from dulcinea.survey import Question, TextQuestion, SelectQuestion, \
     QuestionDatabase, TextAnswer, SelectAnswer, Survey, Surveyable
from dulcinea.user import DulcineaUser

tested_modules = ["dulcinea.survey"]


class SurveyTest (TestScenario):

    class TestSurveyable(Surveyable):

        def __init__(self, question_db=None):
            Surveyable.__init__(self)
            self.question_db = question_db

        def get_new_survey(self):
            return Survey(self.question_db.get_answers([1,2,3]))

    class BadQuestion(Question):
        pass

    def setup(self):
        self.user = DulcineaUser('u')
        self.questions = [
            TextQuestion("text1 title", sub_title="text sub title"),
            SelectQuestion("select1 title", column_headings=['Yes', 'No']),
            SelectQuestion("select2 title",
                           sub_title="check-all-that-apply question",
                           row_headings=["MEMS using micromachining",
                                         "MEMS using bulk micro",
                                         "MEMS using silicon substrate",
                                         "MEMS using silicon substrate",
                                         "MEMS equipment supplier",
                                         "MEMS material supplier",
                                         "MEMS service supplier",
                                         "Other (please specify):"],
                           last_row_has_text=True),
            SelectQuestion("select3 title",
                           sub_title="This example uses a numbered radio set",
                           column_headings=map(str, range(1,10)),
                           row_headings=["Material type",
                                         "Thickness",
                                         "Thickness uniformity",
                                         "Resistivity",
                                         "Flatness",
                                         "Defect level"]),
            SelectQuestion("select4 title",
                           sub_title="This example uses 'range labels'",
                           range_labels = ["Not Important",
                                           "Very Important"],
                           column_headings=map(str, range(1,10)),
                           row_headings=["Material type",
                                         "Thickness",
                                         "Thickness uniformity",
                                         "Resistivity",
                                         "Flatness",
                                         "Defect level"])]

    def check_question_db(self):
        self.test_stmt("self.question_db = QuestionDatabase()")
        for question in self.questions:
            self.test_stmt("self.question_db.add(question)")

    def check_bad_question(self):
        self.test_stmt("b1 = self.BadQuestion('bad')")
        self.test_exc("b1.get_new_answer()", NotImplementedError)

    def check_text_question(self):
         self.test_stmt("t1 = self.question_db.get(1)")
         self.test_val("t1.get_new_answer().__class__", TextAnswer)
         self.test_val("t1.get_title()", "text1 title")
         self.test_val("t1.get_sub_title()", 'text sub title')

    def check_select_question(self):
         self.test_stmt("s1 = self.question_db.get(2)")
         self.test_val("s1.get_new_answer().__class__", SelectAnswer)
         self.test_val("s1.get_title()", "select1 title")
         self.test_val("s1.get_sub_title()", None)
         self.test_val("s1.get_column_headings()", ['Yes', 'No'])
         self.test_val("s1.get_row_headings()", [])
         self.test_val("s1.get_range_labels()", [])
         self.test_stmt("s2 = self.question_db.get(3)")
         self.test_val("s2.get_sub_title()", "check-all-that-apply question")
         self.test_val("s2.get_row_headings()", ["MEMS using micromachining",
                                                 "MEMS using bulk micro",
                                                 "MEMS using silicon substrate",
                                                 "MEMS using silicon substrate",
                                                 "MEMS equipment supplier",
                                                 "MEMS material supplier",
                                                 "MEMS service supplier",
                                                 "Other (please specify):"])
         self.test_stmt("s4 = self.question_db.get(5)")
         self.test_val("s4.get_range_labels()", ["Not Important",
                                                 "Very Important"])

    def check_text_answer(self):
         self.test_stmt("self.t1 = self.question_db.get(1)")
         self.test_val("self.t1.get_new_answer().__class__", TextAnswer)
         self.test_stmt("a1 = self.t1.get_new_answer()")
         self.test_val("a1.get_question()", self.t1)
         self.test_val("a1.get_answer()", None)
         self.test_val("a1.has_answer()", False)
         self.test_stmt("a1.set_answer('the answer')")
         self.test_val("a1.get_answer()", 'the answer')
         self.test_val("a1.has_answer()", True)

    def check_select_answer(self):
         self.test_stmt("self.s1 = self.question_db.get(2)")
         self.test_val("self.s1.get_new_answer().__class__", SelectAnswer)
         self.test_stmt("a2 = self.s1.get_new_answer()")
         self.test_val("a2.get_question()", self.s1)
         self.test_val("a2.get_answer()", None)
         self.test_val("a2.has_answer()", False)
         self.test_stmt("a2.set_answer([1,2,3])")
         self.test_val("a2.get_answer()", [1,2,3])
         self.test_val("a2.has_answer()", True)
         self.test_val("a2.get_other_text()", None)
         self.test_stmt("a2.set_other_text('other')")
         self.test_val("a2.get_other_text()", 'other')

    def check_survey(self):
        self.test_stmt("s1 = Survey(self.question_db.get_answers([1,2,3,4,5]))")
        self.test_stmt("str(s1)")
        self.test_val("len(s1)", 5)
        self.test_val("len(s1.get_questions())", 5)
        self.test_val("len(s1.get_answers())", 5)
        self.test_val("s1.get_timestamp()", None)
        self.test_stmt("s1.set_timestamp()")
        self.test_true("s1.get_timestamp() is not None")
        self.test_val("s1.get_user()", None)
        self.test_stmt("s1.set_user(self.user)")
        self.test_val("s1.get_user().get_id()", 'u')
        self.test_false("s1.has_answers()")
        self.test_stmt("a1 = s1.get_answers()[0]")
        self.test_stmt("a1.set_answer('the answer')")
        self.test_true("s1.has_answers()")
        self.test_stmt("s1.set_answers(s1.get_answers(), self.user)")

    def check_surveyable(self):
        self.test_stmt("sv = self.TestSurveyable(question_db=self.question_db)")
        self.test_val("sv.get_surveys()", [])
        self.test_stmt("sv.add_survey(Survey(self.question_db.get_answers([1,2,3])))")
        self.test_stmt("sv.get_survey(0)")
        self.test_stmt("sv.get_new_survey()")
        self.test_stmt("sv2 = Surveyable()")
        self.test_exc("sv2.get_new_survey()", NotImplementedError)

if __name__ == "__main__":
    (scenarios, options) = parse_args()
    options.show_coverage = 1
    #options.show_coverage_lines = 1
    run_scenarios(scenarios, options)



"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/ui/user/__init__.py $
$Id: __init__.py 23233 2003-11-24 21:08:27Z dbinger $
"""
from quixote import get_request, get_user
from quixote.errors import AccessError
from quixote.html import href
from dulcinea import local
from dulcinea.errors import NotLoggedInError
from dulcinea.ui.user.tiny_login import USERID_COOKIE_NAME
from dulcinea.ui.user.util import about_ssl, enabling_cookies, UserSearchForm, \
     ChangePasswordForm, ContactRegisterForm, profile, get_user_or_act_as
from dulcinea.ui.user.login import LoginForm, NewPasswordForm, \
     delete_user_session, act_as_another_user

_q_exports = ['login', 'logout', 'register', 'edit', 'change_password',
              'profile', 'about_ssl', 'enabling_cookies', 'newpass',
              'insecure_login', 'act_as', 'search']

def get_crumb():
    user = get_user_or_act_as(allow_none=True)
    if user:
        return 'User %s' % user
    return 'Sign in'

def login(request):
    # Only encourage users who we think will log in as admin to use SSL
    user = local.get_user_database().get_user(
        get_request().cookies.get(USERID_COOKIE_NAME))
    form = LoginForm(request, secure=user and user.is_admin())
    return form.handle(request)

def logout(request):
    return delete_user_session(request)

def newpass(request):
    form = NewPasswordForm()
    return form.handle(request)

def register(request, success_links=None):
    form = ContactRegisterForm(request, success_links=success_links)
    return form.handle(request)

edit = register

def insecure_login(request):
    form = LoginForm(request, secure=0)
    return form.handle(request)

def change_password(request):
    if request.session.user is None:
        raise NotLoggedInError(
            "You must be signed in to change your password.")
    form = ChangePasswordForm(request.session.user)
    return form.handle(request)

def search(request):
    user = get_user_or_act_as()
    if not user.is_admin():
        raise AccessError('Only admin users can search for users.')
    return UserSearchForm(
        request, local.get_user_database().get_users(sort=1)).handle(request)

def act_as(request):
    return act_as_another_user(request)

_q_index = login





"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/sort.py $
$Id: sort.py 23444 2004-02-06 15:59:49Z dbinger $

Utility functions for sorting sequences.
"""

import re

def sort(seq):
    """(seq) -> [any]

    Sort 'seq', which must be a sequence object of any type.
    If 'seq' is a list object, it will be sorted in-place and returned.
    Otherwise, 'seq' will be converted to a list, sorted in-place,
    and returned.

    Don't just use 'sort(list)' instead of 'list.sort()'; the latter
    makes it clear that an existing list is being sorted in-place.
    Instead, replace code like this:
      k = dict.keys() ; k.sort()
      values = list(object.get_some_funky_sequence()) ; values.sort()
    with code like this:
      k = sort(dict.keys())
      values = sort(object.get_some_funky_sequence())
    """
    if not isinstance(seq, list):
        seq = list(seq)
    seq.sort()
    return seq

def _sort_and_undecorate(dlist):
    dlist.sort()
    list = [val for (_, val) in dlist]
    return list

def str_sort(seq):
    """(seq) -> []

    Sort 'seq' by the str() of each element.
    """
    dlist = [(str(val), val) for val in seq]
    return _sort_and_undecorate(dlist)

def lexical_sort(seq):
    """(seq) -> []

    Sort 'seq' by the str().lower() of each element.
    """
    dlist = [(str(val).lower(), val) for val in seq]
    return _sort_and_undecorate(dlist)

def attr_sort(seq, attr):
    """(seq, attr : string) -> []

    Sort 'seq' by the attribute 'attr' of each element.
    """
    dlist = [(getattr(val, attr), val) for val in seq]
    return _sort_and_undecorate(dlist)

def lex_attr_sort(seq, attr):
    """(seq, attr : string) -> []

    Sort 'seq' by the str().lower() of the 'attr' attribute of each
    element.
    """
    dlist = [(str(getattr(val, attr)).lower(), val) for val in seq]
    return _sort_and_undecorate(dlist)

def method_sort(seq, method):
    """(seq, method : string) -> []

    Sort 'seq' by the result of calling method 'method' on each element.
    """
    dlist = [(getattr(val, method)(), val) for val in seq]
    return _sort_and_undecorate(dlist)

def function_sort(seq, func):
    """(seq, func : function) -> []

    Sort 'seq' by the result of calling 'func' on each element.
    """
    dlist = [(func(val), val) for val in seq]
    return _sort_and_undecorate(dlist)


number_re = re.compile(r'\s*(\d+)|(\d*\.\d+)\s*')

def natural_sort(seq, strfunc=str):
    """(seq, strfunc : function) -> []

    Sort a list of items in a human friendly way.  Strings are sorted as
    usual, except that decimal integer substrings are compared on their
    numeric value.  For example,

        a < a0 < a1 < a1a < a1b < a2 < a10 < a20

    Strings can contain several number parts:

        x2-g8 < x2-y7 < x2-y08 < x8-y8

    in which case numeric fields are separated by nonnumeric characters.
    Leading spaces are ignored. This works very well for IP addresses from
    log files, for example.

    Numeric substrings with decimal points are treated as floating point.

        1.001 < 1.002 < 1.010 < 1.02 < 1.1 < 1.3

    This function was inspired by the Mac "Natural Order" utility.  It
    does not match the Natural Order algorithm exactly and probably could
    use improvements.
    """
    dlist = []
    for item in seq:
        parts = []
        s = strfunc(item)
        while s:
            m = number_re.search(s)
            if not m:
                parts.append(s)
                break
            else:
                parts.append(s[:m.start()])
                if m.group(1):
                    val = int(s[m.start(1):m.end(1)])
                else:
                    val = float(s[m.start(2):m.end(2)])
                parts.append(val)
                s = s[m.end():]
        dlist.append((filter(None, parts), item))
    return _sort_and_undecorate(dlist)



#!/www/python/bin/python

"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/test/test_permission.py $
$Id: test_permission.py 23445 2004-02-06 16:14:55Z dbinger $

Test for dulcinea.permission.
"""

from sancho.unittest import TestScenario, parse_args, run_scenarios
from dulcinea.permission import PermissionManager

tested_modules = [ "dulcinea.permission" ]

class WaitPass (PermissionManager):

    valid_permissions = {'pass' : 'just pass',
                         'wait' : 'just wait'}


class PermissionManagerTest (TestScenario):

    def check_recursion_control(self):
        self.test_stmt("self.s=s=WaitPass()")
        self.test_stmt("self.r=r=WaitPass()")
        self.test_stmt("self.q=q=WaitPass()")
        self.test_stmt("s.set_permission('pass', r, prerequisite='pass')")
        self.test_stmt("r.set_permission('pass', q, prerequisite='pass')")
        self.test_stmt("q.set_permission('pass', s, prerequisite='pass')")
        self.test_val("s.grants('pass', None)", None)
        self.test_val("r.grants('pass', None)", None)
        self.test_val("q.grants('pass', None)", None)
        self.test_stmt("s.set_permission('pass', s, prerequisite='pass')")
        self.test_val("s.grants('pass', None)", None)
        self.test_val("r.grants('pass', None)", None)
        self.test_val("q.grants('pass', None)", None)

    def check_all(self):
        self.test_stmt("self.s=s=WaitPass()")
        self.test_stmt("self.r=r=WaitPass()")
        self.test_stmt("self.q=q=WaitPass()")
        # None is used in this test as the object for which
        # permissions are applied.  Permissions can be granted to
        # any hashable object.
        self.test_val("s.grants('pass', None)", None)
        self.test_val("s.directly_grants('pass', None)", False)
        self.test_stmt("s.set_permission('pass', None)")
        self.test_stmt("s.set_permission_items('pass', [(None,None)])")

        # None shall pass.  self.s grants 'pass' to None.
        self.test_val("s.grants('pass', None)", [('pass', self.s)])
        # Set that r grants 'wait' if s grants 'pass'
        self.test_stmt("self.r.set_permission('wait', s, prerequisite='pass')")

        # Check get_direct_grantees
        self.test_val("self.r.get_direct_grantees('wait')", [])
        self.test_val("self.s.get_direct_grantees('pass')", [None])

        # r does not grant 'pass'
        self.test_val("r.grants('pass', None)", None)
        # r grants 'wait' to None because self.s grants 'pass' to None.
        self.test_val("r.grants('wait', None)", [('pass', self.s)])
        # Set that q grants 'pass' if r grants 'wait'.
        self.test_stmt("self.q.set_permission('pass', r, prerequisite='wait')")
        # q grants 'pass' to None because self.r grants 'wait' to None,
        # in turn because self.s grants 'pass' to None.
        self.test_val("q.grants('pass', None)",
                      [ ('wait', self.r), ('pass', self.s) ])
        # Set that s does not grant 'pass' to None.
        self.test_stmt("self.s.remove_permission('pass', None)")
        # Now the chain is broken, and q no longer grants 'pass' to None.
        self.test_val("q.grants('pass', None)", None)
        # Set again that s grants 'pass' to None.
        self.test_stmt("self.s.set_permission('pass', None)")
        # The chain is restored.
        self.test_val("q.grants('pass', None)",
                      [ ('wait', self.r), ('pass', self.s) ])
        # Take away r's indirect grant of 'wait'.
        self.test_stmt("self.r.remove_permission('wait', self.s)")
        # The chain is broken again.
        self.test_val("q.grants('pass', None)", None)
        # Check accessor
        self.test_seq("q.get_valid_permissions().keys()", ['pass', 'wait'],
                      match_order=False)
        # Set that r grants 'wait' to 1.
        self.test_stmt("self.r.set_permission('wait', 1)")
        # Check that q grants 'pass' which forces check of r grants 'wait'
        self.test_val("q.grants('pass', None)", None)
        # check get_grants()
        self.test_val("self.r.get_permissions(None)", [])
        self.test_val("self.s.get_permissions(None)", ['pass'])
        self.test_val("self.s.grants('nonsense', None)", None)


if __name__ == "__main__":
    (scenarios, options) = parse_args()
    run_scenarios (scenarios, options)




"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/immutable_list.py $
$Id: immutable_list.py 20912 2003-03-04 16:07:12Z dbinger $

Contains an immutable list class, that can be concatenated with other
lists but can't be mutated in-place.  This should be used to declare
list constants that will truly remain constant and won't be
inadvertantly modified by other code.

"""

class ImmutableList (list):

    """An immutable list class, that can be concatenated with other
    lists but can't be mutated in-place.
    """

    __slots__ = []

    def __setitem__(self, i, item):
        raise TypeError, "ImmutableList doesn't support item assignment"
    def __delitem__ (self, i):
        raise TypeError, "ImmutableList doesn't support item deletion"
    def __setslice__(self, i, j, other):
        raise TypeError, "ImmutableList doesn't support slice assignment"
    def __delslice__(self, i, j):
        raise TypeError, "ImmutableList doesn't support slice deletion"
    def __iadd__(self, other):
        raise TypeError, "ImmutableList doesn't support in-place addition"
    def __imul__(self, other):
        raise TypeError, \
              "ImmutableList doesn't support in-place multiplication"
    def append(self, item):
        raise TypeError, "ImmutableList doesn't support append()"
    def insert(self, i, item):
        raise TypeError, "ImmutableList doesn't support insert()"
    def pop(self, i=-1):
        raise TypeError, "ImmutableList doesn't support pop()"
    def remove(self, item):
        raise TypeError, "ImmutableList doesn't support remove()"
    def reverse(self):
        raise TypeError, "ImmutableList doesn't support reverse()"
    def sort(self, *args):
        raise TypeError, "ImmutableList doesn't support sort()"
    def extend(self, other):
        raise TypeError, "ImmutableList doesn't support extend()"

# class ImmutableList



#!/www/python/bin/python 
"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/bin/expire_session.py $
$Id: expire_session.py 22753 2003-10-15 21:26:17Z nascheme $

Delete sessions whose access time is older than the age passed in as an
argument (in hours).

This script is meant to be run by cron.
"""

import sys, os
from dulcinea import local
from dulcinea.database import get_transaction


def main (prog, args):
    usage = "usage: %s [site] [expire time]" % prog

    if len(args) != 2:
        sys.exit(usage)

    site = os.environ['SITE'] = args[0]
    expiration_hours = float(args[1])

    local.open_database()
    session_mgr = local.get_session_manager()
    expired_count = session_mgr.del_sessions(age=expiration_hours)
    if expired_count:
        get_transaction().commit()

if __name__ == '__main__':
    main(sys.argv[0], sys.argv[1:])



"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/script_util.py $
$Id: script_util.py 21651 2003-05-15 13:56:30Z dbinger $

Various utility functions and classes that are handy when writing good
ol'-fashioned command-line scripts.
"""

import sys, os, time, traceback, pdb

# We only import Option here so that scripts can import it from us,
# without having to import from optik directly.
import optik
from optik import Option
import pdb
from dulcinea.database import get_transaction

# Optik OptionValues instance; set by OptionParser.parse_args(),
# and consulted by various functions here (eg. for 'verbose'
# or 'commit' flags)
_optik_options = None

class OptionParser (optik.OptionParser):
    """OptionParse subclass that provides some standard command-line
    options:
      -v
      --verbose   
      -q, --quiet
      -n, --dry-run

    'verbose' defaults to 1, so scripts should be moderately chatty
    by default.  Use the 'log()' function to emit messages according
    to the user-specified verbosity level.

    Instance attributes: none
    """

    standard_option_list = optik.OptionParser.standard_option_list + [
        Option('-v', action='count', dest='verbose', default=1,
               help="increment verbosity level"),
        Option('--verbose', action='store', type='int', dest='verbose',
               metavar="N", help="set verbosity level to N"),
        Option('-q', '--quiet', action='store_false', dest='verbose',
               help="turn off verbosity (no output except errors)"),
        Option('-n', '--dry-run', action='store_true',
               dest='dry_run', default=0,
               help="don't commit changes to the database"),
        ]


    def parse_args (self, args=None, values=None):
        global _optik_options
        (_optik_options, args) = optik.OptionParser.parse_args(self, args,
                                                               values)
        return (_optik_options, args)


# -- Output utility functions ------------------------------------------

def writenow (msg, file=None):
    if file is None:
        file = sys.stdout
    file.write(msg)
    file.flush()

def warn (msg):
    print >>sys.stderr, "warning: %s" % msg

def error (msg):
    print >>sys.stderr, "error: %s" % msg

def die (msg):
    sys.exit(msg)


# -- Miscellaneous utility functions -----------------------------------

def get_user_info ():
    """Return a "user@hostname" string.
    """
    import socket
    user_id = os.environ.get("USER") or os.environ.get("LOGNAME") or "?"
    host = socket.gethostname()
    return "%s@%s" % (user_id, host)


# -- High-level utility functions --------------------------------------
# (You *must* call 'parse_args()' on some OptionParser instance
# before using either: log() or commit()!)

def log (msg, threshold=1, options=None):
    global _optik_options
    if options is None:
        options = _optik_options
    if options.verbose >= threshold:
        writenow(msg)

def commit (options=None):
    global _optik_options
    if options is None:
        options = _optik_options
    if options.dry_run:
        log("skipping commit\n", threshold=2)
    else:
        log("committing...", options=options)
        get_transaction().commit()
        log("done\n", options=options)

def run ():
    """Run the 'main()' function from the '__main__' module, handling
    some common exceptions in a user-friendly way.
    """
    try:
        import __main__
        __main__.main()
    except (OSError, IOError), err:
        die("%s: %s" % (err.filename, err.strerror))

class Debugger(pdb.Pdb):
    """
    Calling the constructor of this class puts you in the
    interactive debugger.  This is like pdb.post_mortem, except
    that the traceback is printed and the exit command works.
    """
    def __init__ (self):
        pdb.Pdb.__init__(self)
        (exc_type, exc_value, tb) = sys.exc_info()
        up = 0
        if tb is None:
            try:
                raise 
            except:
                (exc_type, exc_value, tb) = sys.exc_info()
            down = 1 
        self.reset()
        while tb.tb_next is not None:
            tb = tb.tb_next
        self.interaction(tb.tb_frame, tb)

    def interaction (self, frame, traceback):
        self.setup(frame, traceback)
        self.curindex = self.curindex - 1
        self.curframe = self.stack[self.curindex][0]
        self.lineno = None
        self.do_explain()
        self.cmdloop()
        self.forget()

    def do_explain (self, arg=None):
        print "\n"
        for stack_entry in self.stack[self.curindex-3:-1]:
            self.print_stack_entry(stack_entry)
        print
        self.do_args(None)
        print
        self.do_list("%s,13" % (self.curframe.f_lineno - 12))

    def help_explain (self):
        print """
Print a slice of the stack, args to this function, and list the list the
section of code being executed.
"""

    def do_exit (self, arg):
        os._exit(1)

    def help_exit (self):
        print """Terminate this process."""

def verbose (arg, _depth=1):
    """Print arg, eval(arg), and then report the time it took."""
    frame = sys._getframe(_depth)
    start = time.time()
    print arg
    result = eval(arg, frame.f_globals, frame.f_locals)
    finish = time.time()
    print '%s completed in %1.3f seconds.' % (arg, finish-start)
    return result

def verbose_catch (arg):
    """
    Like verbose(), but drop into the debugger
    on exceptions.
    """
    try:
        verbose(arg, _depth=2)
    except SystemExit:
        raise
    except:
        print arg, 'FAILED'
        Debugger()

def verbose_main (fun):
    """
    verbose_main parses a command line and calls fun
    with a first argument set to either verbose
    or else to verbose_catch.
    Command line arguments not declared as options here are passed on
    as arguments to fun.
    """
    usage = "usage: %prog [options] [dbfile]"
    parser = OptionParser(usage)
    parser.add_option("-c", "--catch-errors", action="store_true",
                      help=("catch exceptions from update code and "
                            "drop into pdb (default: let it crash"))
    (options, args) = parser.parse_args()
    if options.catch_errors:
        fun(verbose_catch, *args)
    else:
        fun(verbose, *args)





"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/item.py $
$Id: item.py 23460 2004-02-12 22:08:13Z rmasse $
"""
import heapq

from mx import DateTime
from dulcinea.base import DulcineaPersistent, PersistentMapping
from dulcinea.sort import attr_sort
from dulcinea.typeutils import typecheck


class Item(DulcineaPersistent):
    """An Item object implements a mixin API for a unique stored item
    retrievable by creation time.

    Instance attributes:
      title: string
        The title of this item
      timestamp : DateTime
        So Items can be sorted by time.
      key : string
        Some sort of unique ID
    """
    def __init__(self, timestamp=None):
        assert self.__class__ is not Item, "abstract class"
        if timestamp is None:
            self.timestamp = DateTime.now()
        else:
            self.set_timestamp(timestamp)
        self.key = self.title = None

    def __cmp__(self, other):
        return cmp(self.timestamp, other.timestamp)

    def set_key(self, key):
        self.key = key

    def get_key(self):
        return self.key

    def set_timestamp(self, timestamp):
        typecheck(timestamp, DateTime.DateTimeType)
        self.timestamp = timestamp

    def get_timestamp(self):
        return self.timestamp

    def set_title(self, title):
        self.title = title

    def get_title(self):
        return self.title

    def get_subhead(self):
        return None

    def get_user(self):
        return None

    def format_time(self):
        return self.timestamp.strftime('%Y-%m-%d')

class ItemFolder(DulcineaPersistent):
    """Stores a collection of Items and defines an API for storage
    and creation-time-based retrieval.

    Instance attributes:
      items : PersistentMapping { string : Item }
    """

    def __init__(self):
        self.items = PersistentMapping()

    def clear (self):
        self.items.clear()

    def add_item (self, item):
        """(item : Item)
        """
        i = 0
        while 1:
            key =  '%s-%s' % (int(item.timestamp), i)
            if not self.items.has_key(key):
                break
            i += 1
        item.set_key(key)
        self.items[key] = item

    def delete_item (self, item):
        """(item : Item)
        """
        if item.get_key() is None:
            raise KeyError, "Item %r not in any database" % item
        if not self.items.has_key( item.key ):
            raise KeyError, "Item %r not in this database" % item
        del self.items[item.get_key()]

    def get_item (self, key):
        """(key : str) -> Item
        """
        return self.items[key]

    def get_items (self):
        """() -> [Item]
        """
        return self.items.values()

    def get_recent_items(self, N, require_approval=True):
        """(N : int, require_approval : bool = True) -> [Item]

        Return the N most recent items.  If 'require_approval' is true,
        only approved items are returned.
        """
        # XXX items should really be stored in sorted order since access is
        # probably much more frequent than updates
        if N is None:
            result = []
            for item in self.items.values():
                if require_approval and not item.approved:
                    continue
                result.append(item)
            result = attr_sort(result, 'timestamp')
            result.reverse()
        else:
            # Find the N most recent items.
            result = []
            for item in self.items.values():
                if require_approval and not item.approved:
                    continue
                if len(result) < N:
                    heapq.heappush(result, item)
                else:
                    if item > result[0]:
                        heapq.heapreplace(result, item)
            result.sort()
            result.reverse()
        return result



#!/www/python/bin/python

"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/test/test_immutable_list.py $
$Id: test_immutable_list.py 20909 2003-03-04 14:35:33Z dbinger $

Test for immutable lists.
"""

from sancho.unittest import TestScenario, parse_args, run_scenarios
from dulcinea.immutable_list import ImmutableList

tested_modules = ["dulcinea.immutable_list"]

class ImmutableListTest (TestScenario):

    def setup (self):
        self.list = ImmutableList([1,2,3])

    def shutdown (self):
        del self.list

    def check_addition (self):
        "Check that immutable lists can be added to other lists"
        self.test_seq('self.list + [4,5,6]', [1,2,3,4,5,6], match_types=1)
        self.test_stmt('(self.list + [4,5,6]).append(7)')
        self.test_stmt('([4,5,6] + self.list).append(7)')

    def check_failure (self):
        "Check that mutating methods fail"
        self.test_exc('self.list[0] = 0', TypeError)
        self.test_exc('self.list[0:2] = [0]', TypeError)
        self.test_exc('self.list += [4]', TypeError)
        self.test_exc('self.list.append(4)', TypeError)
        self.test_exc('self.list.extend([4,5,6])', TypeError)
        self.test_exc('self.list += [4]', TypeError)
        self.test_exc('self.list *= 4', TypeError)


if __name__ == "__main__":
    (scenarios, options) = parse_args()
    run_scenarios(scenarios, options)



#!/www/python/bin/python

"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/test/test_sort.py $
$Id: test_sort.py 23444 2004-02-06 15:59:49Z dbinger $

Test for dulcinea.sort.
"""

from __future__ import generators

from sancho.unittest import TestScenario, parse_args, run_scenarios
from dulcinea.sort import sort, str_sort, lexical_sort, \
     attr_sort, lex_attr_sort, \
     method_sort, function_sort, natural_sort

tested_modules = [ "dulcinea.sort" ]

class Thing:
    counter = 0

    def __init__(self, name):
        self.name = name
        Thing.counter += 1
        self.id = "T-%03d" % Thing.counter

    def __repr__(self):
        return "<Thing: %s>" % self.id

    def __str__(self):
        return self.name

    def __cmp__(self, other):
        return cmp(self.id, other.id)

    def get_id(self):
        return self.id


def gen(*args):
    for arg in args:
        yield arg


class SortTest (TestScenario):

    def check_sort(self):
        words = ["foo", "BAR", "aBc-123", "blah!", "123-abc"]
        sorted = ["123-abc", "BAR", "aBc-123", "blah!", "foo"]
        self.test_val("sort(gen(*words))", sorted)
        words2 = words[:]
        self.test_val("sort(words2)", sorted)
        self.test_val("sort(words)", sorted)
        self.test_val("words", sorted)

    def check_instances(self):
        thing1 = Thing("bob")
        thing2 = Thing("Tim")
        thing3 = Thing("123-abc")
        things = [thing2, thing1, thing3]
        self.test_val("sort(gen(thing2, thing1, thing3))",
                      [thing1, thing2, thing3])
        self.test_val("str_sort(things)",
                      [thing3, thing2, thing1])
        self.test_val("lexical_sort(gen(thing1, thing2, thing3))",
                      [thing3, thing1, thing2])
        self.test_val("attr_sort(things, 'id')",
                      [thing1, thing2, thing3])
        self.test_val("attr_sort(things, 'name')",
                      [thing3, thing2, thing1])
        self.test_val("lex_attr_sort(things, 'name')",
                      [thing3, thing1, thing2])
        self.test_val("method_sort(gen(thing2, thing1, thing3), 'get_id')",
                      [thing1, thing2, thing3])
        def fun_get_id(x):
            return x.get_id()
        self.test_val("function_sort(gen(thing2, thing1, thing3), fun_get_id)",
                      [thing1, thing2, thing3])

    def check_natural(self):
        nums = [1, 5, 9, 10, 15, 105]
        self.test_val("str_sort(gen(*nums))", [1, 10, 105, 15, 5, 9])
        nums = map(str, nums)
        self.test_val("sort(nums[:])", ["1", "10", "105", "15", "5", "9"])
        self.test_val("natural_sort(nums)", nums)

        # Check that the promises made in the docstring hold.
        l = ["a1b", "a1", "a20", "a", "a0", "a1a", "a10", "a2"]
        self.test_val("natural_sort(l)",
                      ["a", "a0", "a1", "a1a", "a1b", "a2", "a10", "a20"])
        l = ["x2-y7", "x8-y8", "x2-g8", "x2-y08"]
        self.test_val("natural_sort(l)",
                      ["x2-g8", "x2-y7", "x2-y08", "x8-y8"])
        l = ["1.02", "1.001", "1.3", "1.002", "1.1", "1.010"]
        self.test_val("natural_sort(l)",
                      ["1.001", "1.002", "1.010", "1.02", "1.1", "1.3"])


if __name__ == "__main__":
    (scenarios, options) = parse_args()
    run_scenarios(scenarios, options)



"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/ui/form/__init__.py $
$Id: __init__.py 23198 2003-11-19 20:17:27Z rmasse $
"""
from dulcinea.ui.form import string_list_widget, address_widget, user_widget, \
     date_widget



"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/persistent_list.py $
$Id: persistent_list.py 21569 2003-05-09 00:31:47Z abenard $

Provides the PersistentList class, an analogue to ZODB's PersistentMapping
that ZODB forgot.  Hopefully this module will go away someday, to be
replaced by ZODB.PersistentList.
"""

import UserList
from dulcinea.base import DulcineaPersistent


class PersistentList(DulcineaPersistent, UserList.UserList):
    """Wrapper class for lists that sets ._p_changed correctly.

    Instance attributes:
      data : [any]
    """

    def __init__(self, initlist=None):
        UserList.UserList.__init__(self, initlist)

    def copy(self):
        return self.__class__(self.data)

    def __setitem__(self, i, item):
        self._p_changed = 1
        self.data[i] = item

    def __delitem__(self, i):
        self._p_changed = 1
        del self.data[i]

    def __setslice__(self, i, j, other):
        self._p_changed = 1
        i = max(i, 0); j = max(j, 0)
        if isinstance(other, UserList.UserList):
            self.data[i:j] = other.data
        elif isinstance(other, type(self.data)):
            self.data[i:j] = other
        else:
            self.data[i:j] = list(other)

    def __delslice__(self, i, j):
        self._p_changed = 1
        i = max(i, 0); j = max(j, 0)
        del self.data[i:j]

    def append(self, item):
        self._p_changed = 1
        self.data.append(item)

    def insert(self, i, item):
        self._p_changed = 1
        self.data.insert(i, item)

    def pop(self, i=-1):
        self._p_changed = 1
        return self.data.pop(i)

    def remove(self, item):
        self._p_changed = 1
        self.data.remove(item)

    def reverse(self):
        self._p_changed = 1
        self.data.reverse()

    def sort(self, *args):
        self._p_changed = 1
        apply(self.data.sort, args)

    def extend(self, other):
        self._p_changed = 1
        if isinstance(other, UserList.UserList):
            self.data.extend(other.data)
        else:
            self.data.extend(other)

    def __repr__(self):
        return '<%s at %x: %d item(s)>' % (self.__class__.__name__,
                                           id(self), len(self.data))

    def __cmp__(self, other):
        # ExtensionClass does not support richcomps.  UserList only provides
        # richcomps. :-(
        return cmp(self.data, other)


# class PersistentList



#!/www/python/bin/python
"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/test/test_calendar.py $
$Id: test_calendar.py 23278 2003-12-08 22:14:29Z dbinger $
"""

from mx.DateTime import DateTime, TimeDelta, now
from sancho.unittest import TestScenario, parse_args, run_scenarios
from dulcinea.user import DulcineaUser as User
from dulcinea.calendar import Calendar, Reservation, Reservable, Resources, \
     Resource

tested_modules = ["dulcinea.calendar"]

class Test (TestScenario):

    def check_a(self):
        self.test_stmt('self.calendar = Calendar()')
        self.test_stmt('self.block1 = Reservation()')
        self.test_exc('self.calendar.add(self.block1)', TypeError)
        title = 'test1'
        self.test_stmt('self.block1.set_title(title)')
        self.test_val('self.block1.get_title()', title)
        self.test_stmt('self.block1.set_description(title)')
        self.test_val('self.block1.get_description()', title)
        resource = Resource()
        reserved = [resource]
        self.test_stmt('self.block1.set_reserved(reserved)')
        self.test_val('self.block1.get_reserved()', reserved)
        users = []
        self.test_stmt('self.block1.set_users(users)')
        self.test_val('self.block1.get_users()', users)
        start1 = DateTime(1)
        self.test_stmt('self.block1.set_start_time(start1)')
        self.test_val('self.block1.get_start_time()', start1)
        end1 = start1 + 1
        self.test_stmt('self.block1.set_end_time(end1)')
        self.test_val('self.block1.get_end_time()', end1)

        self.test_stmt('self.calendar.add(self.block1)')
        self.test_val('self.calendar.get(self.block1.get_key())', self.block1)
        self.test_val("self.calendar.get('', None)", None)
        self.test_val('self.calendar.get_mapping()',
                      { self.block1.get_key():  self.block1 })

        self.test_stmt('self.reservable = Reservable()')
        self.test_stmt('self.reservable.set_active(True)')
        self.test_val('self.reservable.is_active()', True)
        title = 'room 1'
        self.test_stmt('self.reservable.set_title(title)')
        self.test_val('self.reservable.get_title()', title)
        self.test_stmt('self.block1.set_reserved([self.reservable])')

        user = User()
        self.test_val('self.reservable.allows_control(user)', None)
        self.test_val('self.reservable.allows_reserve(user)', None)
        class AdminUser(User):
            def is_admin(self):
                return True
        admin_user = AdminUser()
        self.test_val('self.block1.can_modify(None)', False)
        self.test_val('self.block1.can_modify(admin_user)', True)

        self.test_val('self.block1.can_modify(user)', False)
        self.reservable.set_permission('control', user)
        self.test_val('self.block1.can_modify(user)', True)

        user2 = User()
        self.test_stmt('self.block1.set_users([user2])')
        self.test_val('self.block1.can_modify(user2)', True)

        self.test_stmt('Resources()')

        time_increment = TimeDelta(hours=2)
        start_time = start1
        end_time = start_time + 3
        start_time_of_day = TimeDelta(hours=8)
        end_time_of_day = TimeDelta(hours=18)
        resource = Resource()
        reservation = Reservation()
        self.test_stmt('reservation.set_title("")')
        self.test_stmt('reservation.set_description("")')
        self.test_stmt('reservation.set_users([])')
        self.test_stmt('reservation.set_reserved([resource])')
        self.test_stmt('reservation.set_start_time(start_time + start_time_of_day)')
        self.test_stmt(
            'reservation.set_end_time(start_time + start_time_of_day + '
            '2 * time_increment)')
        self.test_stmt('reservation.set_reserved([resource])')
        self.test_stmt('self.calendar.add(reservation)')
        self.test_stmt('self.calendar.get_table_map('
                       'resource, time_increment, start_time, end_time, '
                       'start_time_of_day, end_time_of_day)')


if __name__ == "__main__":
    (scenarios, options) = parse_args()
    run_scenarios(scenarios, options)



"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/ui/item.py $
$Id: item.py 23235 2003-11-24 21:32:40Z rmasse $

Contains a skeletal class for submitting and updating posted items.
"""

from quixote import get_user
from quixote.errors import AccessError
from quixote.form import Form
from quixote.html import htmltext
from dulcinea import local_ui
from dulcinea.errors import NotLoggedInError

class AbstractItemForm (Form):
    """Skeleton for writing forms that add a posted item

    Subclasses must implement four methods:

    make_new_item() :
      Creates a new item of the right class.
    create_widgets(request) :
      Creates the widgets for the form.
    check_form (request, form_data) :
      Check the form data's validity, and sets self.error accordingly.
    add_item (request) :
      Add a new item, available as self.item, to the appropriate database.
      This method should return a PTL page, which will be shown to the user.
    """

    def __init__ (self, request, item=None):
        Form.__init__(self)
        if item is None:
            item = self.item = self.make_new_item()
            self.new_item = True
        else:
            self.item = item
            self.new_item = False

        self.create_widgets(request)

        if self.new_item:
            self.add_submit_button("Preview", "Preview")
            self.add_submit_button("Submit", "Submit")
            self.add_cancel_button("Cancel", request.get_path(1) + '/')
        else:
            title = self.item.get_title()
            if len(title) > 40:
                title = title[:40] + " ..."
            self.add_submit_button("Update", "Update")
            self.add_cancel_button("Return to %s" % title,
                                   request.get_path(1) + '/')


    def _render_error_notice (self, request):
        if (len(self.error) > 1 or
            len(self.error) == 1 and not self.error.has_key('dummy_field')):
            return Form._render_error_notice(self, request)
        else:
            return ""

    def process (self, request):
        form_data = Form.process(self, request)
        self.check_form(request, form_data)

        # In preview mode and in editing mode, we need to set some
        # error so that the .action() method isn't called and we
        # just return to the form.  This error will never be
        # displayed.
        if (request.form.has_key('Preview') or
            request.form.has_key('Update') ):
            self.error['dummy_field'] = 'dummy message'

        return form_data


    def action (self, request, submit, form_data):
        # Submit the item
        if self.new_item:
            return self.add_item(request)
        else:
            return request.redirect(request.get_path(2) + '/')

    # Subclasses must implement the following methods.
    def make_new_item (self):
        raise RuntimeError('Abstract make_new_item() method called')

    def create_widgets (self, request):
        raise RuntimeError('Abstract create_widgets() method called')

    def check_form (self, request, form_data):
        raise RuntimeError('Abstract check_form() method called')

    def add_item (self, request):
        raise RuntimeError('Abstract add_item() method called')

class ItemDeleteForm(Form):

    def __init__(self, request, item, item_db):
        """(item : Item, item_db : ItemFolder)
        """
        Form.__init__(self)
        self.item = item
        self.item_db = item_db
        self.add_submit_button("Delete", "Delete")
        self.add_cancel_button("Cancel", request.get_path(1) + '/')

    def render(self, request, action_url):
        title = "Delete %s" % self.item.get_title()
        s = htmltext(local_ui.header(title=title))
        s += htmltext('<p>Delete item "%s"?</p>' % self.item.get_title())
        s += Form.render(self, request, action_url)
        s += local_ui.footer(title)
        return s

    def action(self, request, submit, form_data):
        self.item_db.delete_item(self.item)
        return request.redirect(request.get_path(2) + '/')

def delete_item(request, item, item_db, name):
    user = request.session.user
    if user and (user is item.get_user() or user.is_admin()):
        form = ItemDeleteForm(request, item, item_db)
        return form.handle(request)
    else:
        raise AccessError, ("Attempt to delete %s item by non-owner "
                            "or a non-admin user." % name)

def edit_item(request, item, edit_form, name):
    user = get_user()
    if not user:
        raise NotLoggedInError("You must be logged in to edit items.")
    if user is item.get_user() or user.is_admin():
        form = edit_form(request, item)
        return form.handle(request)
    else:
        raise AccessError, ("Attempt to edit %s item by non-owner "
                            "or a non-admin user." % name)



#!/www/python/bin/python
"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/test/test_keyed.py $
$Id: test_keyed.py 23278 2003-12-08 22:14:29Z dbinger $
"""

from sancho.unittest import TestScenario, parse_args, run_scenarios
from dulcinea.keyed import Keyed, KeyedMap

tested_modules = ["dulcinea.keyed"]

class Test (TestScenario):

    def check_a(self):
        self.test_stmt('self.map = KeyedMap()')
        self.test_stmt('self.a = Keyed()')
        self.test_val('self.a.get_key()', None)        
        self.test_stmt('self.map.add(self.a)')
        self.test_exc('self.map.add(self.a)', AssertionError)
        self.test_stmt('self.b = Keyed()')
        self.test_stmt('self.b.junk = 1')
        self.test_exc('self.map.add(self.b)', TypeError)
        self.test_exc('self.map.add(1)', TypeError)        
        self.test_val('self.a.get_key()', 1)
        self.test_val('self.map.get_mapping().items()', [(1, self.a)])
        self.test_val('self.map.get(1)', self.a)

if __name__ == "__main__":
    (scenarios, options) = parse_args()
    run_scenarios(scenarios, options)



#!/www/python/bin/python

"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/tools/zodb_census.py $
$Id: zodb_census.py 20912 2003-03-04 16:07:12Z dbinger $

Inspect every object in a ZODB and count how many times each type
occurs.  (Note that each ExtensionClass is a separate type, so
we'll get a class-by-class breakdown from this.)
"""

import sys
from dulcinea.script_util import writenow

object_count = {}                       # maps type name to count

def main(db, default_dbspec):
    if len(sys.argv) > 1:
        dbspec = sys.argv[1]
    else:
        dbspec = default_dbspec

    writenow("opening database...")
    db.open(dbspec)
    writenow("\n")

    for obj in db.iterate():
        typename = type(obj).__name__
        if object_count.has_key(typename):
            object_count[typename] += 1
        else:
            object_count[typename] = 1L

    writenow("census completed\n")

    typenames = object_count.keys()
    typenames.sort()
    print "objects seen by type:"
    for name in typenames:
        print "%-25.25s %10d" % (name, object_count[name])




"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/database.py $
$Id: database.py 23388 2004-02-03 00:14:30Z rmasse $

Provides the ObjectDatabase class, for initializing and accessing a ZODB
containing a set of database roots.
"""
from __future__ import generators

import sys, os, atexit, struct, time
from ZODB.DB import DB
from dulcinea.errors import ConflictError

# provide a uniform location for get_transaction
try:
    from Transaction import get_transaction # ZODB4 and newer
except ImportError:
    from __builtin__ import get_transaction # old ZODB, yuck


def pack_oid (oid_i):
    """pack_oid(oid_i : long|int) -> string

    Pack an integer or long int into an 8-byte OID string, eg.
    0x0001020304050607L -> "\0\1\2\3\4\5\6\7".
    """
    return struct.pack(">LL",
                       (oid_i & 0xffffffff00000000L) >> 32,
                       (oid_i & 0x00000000ffffffffL))

def unpack_oid (oid_s):
    """unpack_oid(oid_s : string) -> long

    Unpack an 8-byte OID string to a long int, eg.
    "\0\1\2\3\4\5\6\7" -> 0x0001020304050607L
    """
    (hi, lo) = struct.unpack(">LL", oid_s)     # two 32-bit integers
    return (hi << 32) | lo                     # a single 64-bit integer

def format_oid (oid):
    """format_oid(oid : long|int|string) -> string

    Format an OID (either a packed 8-byte string or an integer) to
    human readable form, eg.
      "\0\1\2\3\4\5\6\7" -> "0001020304050607" or
      0x0001020304050607L -> "0001020304050607"
    """
    if isinstance(oid, str):
        oid = unpack_oid(oid)
    return "%016x" % oid

def parse_oid (oid):
    """parse_oid(oid : string) -> string

    Turn a human-readable OID string into a packed 8-byte string, eg.
      "0001020304050607" -> "\0\1\2\3\4\5\6\7"
    """
    return pack_oid(long(oid, 16))


class ObjectDatabase:

    """Class used to initialize, access, and close a ZODB containing a set of
    root objects.
    
    NOTE: Care must be taken to ensure that only one instance exists for each
    ZODB database.

    Instance attributes:
      _dbspec : string
        the connection specification for this database
      _current_root_accessors : [str]
      storage : ZODB.BaseStorageBaseStorage
      database : ZODB.DB.DB
      connection : ZODB.Connection.Connection
      root : any
    """

    def __init__ (self):
        """ObjectDatabase()

        Creates a new ObjectDatabase object.
        """
        self._dbspec = None
        self.storage = None
        self.database = None
        self.connection = None
        self.root = None
        self._current_root_accessors = []
        atexit.register(self.close) # XXX is this really necessary?

    def open(self, dbspec):
        assert self.root is None, 'database already open'
        self._dbspec = dbspec
        if dbspec.find(":") != -1:
            try:
                (storage_type, args) = dbspec.split(":")
            except ValueError:
                raise ValueError, ("dbspec may have at most one "
                                   "colon: '%s' invalid" % dbspec)
            args = tuple(args.split(","))
        else:
            storage_type = dbspec
            args = ()

        if storage_type == "file":
            (filename,) = args
            (self.database, self.connection) = self._open_file(filename)

        elif storage_type == "dbm":
            (filename,) = args
            (self.database, self.connection) = self._open_dbm(filename)

        elif storage_type == "gdbm":
            (filename,) = args
            (self.database, self.connection) = self._open_gdbm(filename)

        elif storage_type == "berkeley":
            (filename,) = args
            (self.database, self.connection) = self._open_berkeley(filename)

        elif storage_type == "client":
            try:
                host, port = args
                port = int(port)
            except ValueError:
                raise ValueError("invalid dbspec: must be "
                                 "'client:<host>,<port>'" % dbspec)
            (self.database, self.connection) = self._open_client((host, port))

        elif storage_type == "empty":
            # leave database and connection as None
            self.root = {}

        else:
            raise ValueError, "invalid storage type '%s'" % storage_type

        if self.connection is not None:
            self.root = self.connection.root()

        if self.database is not None:
            # increase the maximum number of objects cached
            self.database.setCacheSize(10000)

        self.update_root_accessors()


    def close(self):
        "Close the database connection"
        self._dbspec = None
        if self.connection is not None:
            self.connection.close()
            self.connection = None
        if self.database is not None:
            self.database.close()
            self.database = None
        if self.storage is not None:
            self.storage.close()
            self.storage = None
        self.root = None

    def __repr__ (self):
        return '<%s: %s>' % (self.__class__.__name__, self._dbspec)

    # Accessors for the ZODB objects
    def get_storage (self):    return self.storage
    def get_connection (self): return self.connection
    def get_database (self):  return self.database
    def get_root (self):       return self.root

    def get_cache_size (self):
        """() -> current_size:int, nominal_size:int
        Return the number of objects currently in the ZODB cache,
        and the size of the cache.
        """
        return (self.database.cacheDetailSize()[0]['size'],
                self.database.getCacheSize())

    def pack(self):
        self.database.pack(time.time())

    def set_cache_size (self, size):
        """(int)
        Set the maximum cache size.
        """
        if self.database is not None:
            self.database.setCacheSize(size)

    def _open_file (self, filename):
        """Open 'filename' as a FileStorage, and then open a ZODB database
        around that.  Return a (database, connection) tuple containing the
        DB instance and the Connection instance that results from calling
        its 'open()' method."""

        from ZODB.FileStorage import FileStorage
        created = (not os.path.isfile(filename))

        self.storage = FileStorage(filename)
        if created:
            os.chmod(filename, 0664)            # ensure group writeable
            os.chmod(filename + ".lock", 0664)  # gag! retch!
            os.chmod(filename + ".tmp", 0664)   # ...and puuuuke!
        db = DB(self.storage)
        return (db, db.open())

    def _open_dbm (self, filename):
        """Open 'filename' as a dbmStorage, and then open a ZODB database
        around that.  Return a (database, connection) tuple containing the
        DB instance and the Connection instance that results from calling
        its 'open()' method."""

        from ZODB.dbmStorage import anydbmStorage

        self.storage = anydbmStorage(filename, flag='c')
        db = DB(self.storage)
        return (db, db.open())

    def _open_gdbm (self, filename):
        """Open 'filename' as a gdbmStorage, and then open a ZODB database
        around that.  Return a (database, connection) tuple containing the
        DB instance and the Connection instance that results from calling
        its 'open()' method."""

        from ZODB.dbmStorage import gdbmStorage

        self.storage = gdbmStorage(filename, flag='c')
        db = DB(self.storage)
        return (db, db.open())

    def _open_berkeley (self, filename):
        raise NotImplementedError, \
              "BerkeleyStorage interface not implemented yet"

    def _open_client (self, location):
        """Open 'location' (a (hostname, port_number) tuple) as a ZEO
        ClientStorage, and then open a ZODB database around that.  Return
        a (database, connection) tuple."""

        host, port = location
        if host == "":
            # If the specified hostname is the empty string, then
            # 'localhost' is used.
            location = ('localhost', port)

        from ZEO.ClientStorage import ClientStorage
        self.storage = ClientStorage(location, var='/var/tmp',
                                     wait_for_server_on_startup=0)
        db = DB(self.storage)
        return (db, db.open())

    def get_root_object (self, name):
        assert self.root is not None, "database not initialized"
        if not self.root.has_key(name):
            raise ValueError, "invalid database root '%s'" % name
        return self.root[name]

    def _add_root_accessor(self, name):
        if hasattr(self, name):
            raise ConflictError, 'attribute %s already exists' % name
        setattr(self, name, self.root[name])
        self._current_root_accessors.append(name)

    def init_root (self, name, modulename, klassname):
        __import__(modulename)
        module = sys.modules[modulename]
        klass = getattr(module, klassname)
        assert name not in self.root.keys(), name
        self.root[name] = klass()
        self._add_root_accessor(name)

    def update_root_accessors (self):
        for name in self._current_root_accessors:
            if hasattr(self, name):
                delattr(self, name)
        for name in self.root.keys():
            self._add_root_accessor(name)

    def get_root_names (self):
        return self.root.keys()

    def iterate (self, verbose=-1, status_every=0x800):
        """
        Iterate over all OIDs in a ZODB database, yielding every object
        found.

        If verbose is true or it is -1 and stdout is a tty then iterate()
        prints a status line to stdout every status_every OIDs (default
        0x800, or 2048).

        Note that this method currently only works for FileStorage
        connections.
        """
        if not hasattr(self.storage, "_index"):
            raise SystemExit, "must use FileStorage when using iterate()"

        if verbose and not sys.stdout.isatty():
            # no status reports when not a TTY
            status_every = 0

        def write_status (oid, count, newline=0):
            sys.stdout.write("\rOID: %s (objects seen: %d)" %
                             (format_oid(oid), count))
            if newline:
                sys.stdout.write("\n")
            sys.stdout.flush()

        count = 0
        for oid in self.storage._index.keys():
            count += 1
            if status_every and (count % status_every) == 0:
                write_status(oid, count)
            yield self.connection[oid]

        if status_every:
            write_status(oid, count, newline=1)





#!/www/python/bin/python
"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/mems/lib/lib/test/test_comment.py $
$Id: test_comment.py 23256 2003-12-02 18:38:47Z dbinger $
"""

from mx import DateTime
from sancho.unittest import TestScenario, parse_args, run_scenarios
from dulcinea.comment import Comment
from dulcinea.user import DulcineaUser

tested_modules = [ "dulcinea.comment" ]

class CommentTest (TestScenario):

    def check_comment (self):
        user = DulcineaUser("jblow")
        t1 = DateTime.now()
        c = Comment("Hello", user)
        self.test_val("c.get_text()", "Hello")
        self.test_val("c.get_user()", user)
        t2 = DateTime.now()
        self.test_val("c.text", "Hello")
        self.test_val("c.user", user)
        self.test_true("t1 <= c.get_timestamp() <= t2")
        self.test_val("str(c)", "%s at %s" % (user, c.timestamp))
        d = Comment("H", user)
        self.test_true("d <= c")

if __name__ == "__main__":
    (scenarios, options) = parse_args()
    run_scenarios (scenarios, options)



"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/site_util.py $
$Id: site_util.py 22983 2003-11-05 01:53:27Z dbinger $

Provides access to site configuration data.
"""
# Change this if you need a different config file.

SITE_CONF = '/www/conf/site.conf'

import sys, os, re, socket
import ConfigParser

_ptl_enabled = False # delay until needed since it's expensive

class SiteConfig:
    """This is a ConfigParser-like class.  It allows settings for specific
    hosts to be overridden.  For example, if the site.conf contains

        [goodsite]
        mode: devel

        [goodsite darkstar]
        mode: live

    then config.get("goodsite", "mode") will return "devel" on all hosts
    except on "darkstar", where it will return "live".
    """

    def __init__ (self):
        config = ConfigParser.ConfigParser()
        config.readfp(open(SITE_CONF))
        self._defaults = {} # { name : value }
        self._options = {} # { (site, name) : value }
        self._sites = [site for site in config.sections()
                       if site.find(' ') == -1]
        # [DEFAULT]
        for name, value in config.defaults().items():
            for site in self._sites:
                self._defaults[name] = value
        # [site]
        for site in self._sites:
            for name in config.options(site):
                self._options[(site, name)] = config.get(site, name)
        # [site host]
        host = socket.gethostname()
        for site in self._sites:
            section = site + ' ' + host
            if config.has_section(section):
                for name in config.options(section):
                    self._options[(site, name)] = config.get(section, name)

    # XXX should really come up with a nicer interface.  The ConfigParser one
    # is pretty ugly.

    def defaults (self):
        return self._defaults

    def has_option (self, site, name):
        return (site, name) in self._options

    def get (self, site, name, fallback=None):
        if (site, name) in self._options:
            return self._options[(site, name)]
        else:
            return self._defaults.get((site, name), fallback)

    def list_sites (self):
        return self._sites

_config = SiteConfig()

def get_config ():
    return _config

def list_sites ():
    return _config.list_sites()

def get_hostname ():
    return socket.gethostbyname_ex(socket.gethostname())[0]

def is_staging (site):
    return _config.get(site, 'mode') == 'staging'

def is_live (site):
    return _config.get(site, 'mode') == 'live'

def any_live_sites ():
    """Returns True if any sites are live."""
    for site in list_sites():
        if is_live(site):
            return True
    else:
        return False

def _get_local_interfaces():
    """Return a list of local IP addresses for this machine."""
    # Currently a Linux specific hack. :-(
    addrs = []
    p = os.popen("/sbin/ifconfig")
    for line in p.readlines():
        m = re.search(r'inet (addr:)?(\d+\.\d+\.\d+\.\d+)', line)
        if m:
            addrs.append(m.group(2))
    return addrs

local_interfaces = _get_local_interfaces()

def parse_address(s):
    ip, port = s.strip().split(':')
    if not ip[0].isdigit():
        # not an IP address, resolve it
        ip = socket.gethostbyname(ip)
    return ip, port and int(port)

def is_local (address):
    ip, port = parse_address(address)
    return ip in local_interfaces

def get_dbspec (site=None):
    if site is None:
        site = os.environ.get('SITE', None)
    site_config = get_config()
    if site_config.has_option(site, 'zeo-address'):
        zeo_address = site_config.get(site, 'zeo-address')
        return 'client:%s,%s' % parse_address(zeo_address)
    return None

def get_root_exports (site=None):
    if site is None:
        site = os.environ.get('SITE', None)
    return get_config().get(site, 'root-exports').split()
    
def get_cache_size (site=None, default=10000):
    if site is None:
        site = os.environ.get('SITE', None)
    return int(get_config().get(site, 'cache-size', 10000))

_email_pat = re.compile(r'^(.*)\s+<(.*)>$')

def get_administrator_address(site=None):
    if site is None:
        administrator = _config.defaults().get('administrator')
    else:
        administrator = _config.get(site, 'administrator')
    m = _email_pat.match(administrator)
    if not m:
        raise ValueError, '"administrator" in site.conf missing real name'
    realname = m.group(1)
    email = m.group(2)
    return (email, realname)

def get_pid_file_name (daemon, site=None):
    name = '%s.pid' % daemon
    if site:
        name = site + '-' + name
    pid_dir = get_config().defaults().get('var-directory')
    return os.path.join(pid_dir, name)

def write_pid_file (daemon, site=None):
    file = open(get_pid_file_name(daemon, site), 'w')
    file.write(str(os.getpid()))
    file.close()

def change_uid_gid(uid, gid=None):
    "Try to change UID and GID to the provided values"
    # This will only work if this script is run by root.

    # Try to convert uid and gid to integers, in case they're numeric
    import pwd, grp
    try:
        uid = int(uid)
        default_grp = pwd.getpwuid(uid)[3]
    except ValueError:
        uid, default_grp = pwd.getpwnam(uid)[2:4]

    if gid is None:
        gid = default_grp
    else:
        try:
            gid = int(gid)
        except ValueError:
            gid = grp.getgrnam(gid)[2]

    os.setgid(gid)
    os.setuid(uid)


def ensure_uid_gid_not_root ():
    if os.getuid() == 0:
        uid = get_config().defaults().get('daemon-uid')
        change_uid_gid(uid)
    assert os.getuid() != 0

def log_to (log_file_name):
    log = open(log_file_name, "a", 1)
    os.dup2(log.fileno(), 1)
    os.dup2(log.fileno(), 2)
    os.close(0)

def _get_servername (site, mode):
    config = ConfigParser.ConfigParser()
    config.readfp(open(SITE_CONF))
    for section in config.sections():
        if section.split()[0] == site:
            if (config.has_option(section, 'mode') and
                config.get(section, 'mode') == mode):
                if config.has_option(section, 'servername'):
                    return config.get(section, 'servername')

def live_servername (site):
    return _get_servername(site, 'live')

def staging_servername (site):
    return _get_servername(site, 'staging')

def fetch_database (site, destination=None, source_file=None):
    """
    Pull a copy of the live database for site to the local
    machine using scp (using the daemon-uid on
    the remote host).
    If destination is given, it is the name of local file.
    If source file is given, it is a file name
    in the var-directory of the remote host.
    If neither of the keyword arguments are provided, the
    database is pulled from the standard place on the remote machine
    and put in the standard place on the local machine.
    The standard place is <site>.fs in the var-directory.
    """
    config = get_config()
    source_file = os.path.join(config.get(site, 'var-directory'),
                               source_file or ("%s.fs" % site))
    remote = (config.get(site, 'daemon-uid') + '@'
              + live_servername(site) + ':'
              + source_file)
    if not destination:
        destination = os.path.join(config.get(site, 'var-directory'),
                                   "%s.fs" % site)
    cmd = 'scp -C -p %s %s' % (remote, destination)
    print cmd
    os.system(cmd)
    return 'file:' + destination


class LazyModule:
    """A module-like object that is used for the 'local' and 'local_ui'
    modules.  It is necessary because we want to delay importing the site
    specific local modules until they are actually needed.
    """
    def __init__(self, module_name):
        self._loaded = False
        self.__name__ = module_name

    def __getattr__(self, name):
        if not self._loaded and not name.startswith('_'):
            site = os.environ.get('SITE')
            if site is None:
                raise RuntimeError, ("SITE must be set before "
                                     "accessing '%s' module" % self.__name__)
            module_name = site + '.' + self.__name__
            __import__(module_name)
            for n, v in vars(sys.modules[module_name]).items():
                if not n.startswith('_'):
                    setattr(self, n, v)
            self._loaded = True
        try:
            return self.__dict__[name]
        except KeyError:
            raise AttributeError, name

def setup_local_modules():
    """Create 'local' and 'local_ui' module-like objects.  Import of the actual
    site specific modules will be delayed until an attribute is accessed.
    """
    return LazyModule('local'), LazyModule('local_ui')



"""$URL$
$Id$
"""

from urlparse import urlsplit, urlunsplit
from mx import DateTime
from dulcinea.item import Item, ItemFolder
from dulcinea.base import DulcineaPersistent
from dulcinea.spec import any
from dulcinea.typeutils import typecheck

class LinkItem(Item):
    """
    Instance attributes:
       split_url : [ string ]
          urlsplited URL to redirect to
    """

    split_url_is = any([ str ], None)

    def __init__(self, timestamp=None):
        Item.__init__(self, timestamp=timestamp)
        self.split_url = None

    def set_url(self, url):
        typecheck(url, str)
        split_url = urlsplit(url)
        assert split_url[0] in ('ftp', 'http', 'https'), (
            "unknown URL scheme '%s'" % split_url[0])
        self.split_url = split_url
        self.timestamp = DateTime.now()

    def get_url(self):
        return urlunsplit(self.split_url)


LinkFolder = ItemFolder

def sort_reverse_by_date(link1, link2):
    # Sort links cronologically with most recent first
    return cmp(link2.get_timestamp(), link1.get_timestamp())




"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/fm/lib/time_block.py $
$Id: time_block.py 22694 2003-10-06 15:57:51Z dbinger $
"""

from mx.DateTime import DateTimeType
from dulcinea.base import DulcineaPersistent
from dulcinea.keyed import Keyed, KeyedMap
from dulcinea.permission import PermissionManager
from dulcinea.sort import method_sort
from dulcinea.user import DulcineaUser as User
from dulcinea.spec import require

class Titled:
    """
    Instance attributes:
      title : str
      description : str

    """
    title_is = str
    description_is = str

    def __init__(self):
        self.title = None
        self.description = None

    def get_title(self):
        return self.title

    def set_title(self, title):
        require(title, self.title_is)
        self.title = title

    def get_description(self):
        return self.description

    def set_description(self, description):
        require(description, self.description_is)
        self.description = description


class Reservable(DulcineaPersistent, PermissionManager, Keyed, Titled):

    """
    Instance attributes:
      active : bool
    """

    active_is = bool

    valid_permissions = {
        'manage_permissions' : 'able to manage permissions of this item ',
        'control' : 'able to change any reservations of this item',
        'reserve' : 'able to make a reservation of this item',
        }

    def __init__(self):
        Keyed.__init__(self)
        Titled.__init__(self)
        PermissionManager.__init__(self)
        self.active = True

    def set_active(self, value):
        self.active = value

    def is_active(self):
        return self.active

    def allows_reserve(self, user):
        return self.grants('reserve', user)

    def allows_control(self, user):
        return self.grants('control', user)


class Resource (DulcineaPersistent, Reservable):
    pass


class Resources (KeyedMap):

    def __init__(self):
        KeyedMap.__init__(self, Resource)


class Reservation (DulcineaPersistent, Keyed, Titled):
    """
    Instance attributes:
      start_time : DateTime
      end_time : DateTime
      reserved: [ Reservable ]
      users : [ User ]
    """

    start_time_is = DateTimeType
    end_time_is = DateTimeType
    reserved_is = [Reservable]
    users_is = [User]

    def __init__(self):
        Keyed.__init__(self)
        Titled.__init__(self)
        self.start_time = None
        self.end_time = None
        self.reserved = None
        self.users = None

    def __str__ (self):
        return "%s: %s" % (self.key, self.start_time)

    def set_start_time (self, time):
        require(time, self.start_time_is)
        self.start_time = time

    def set_end_time (self, time):
        require(time, self.start_time_is)
        assert time >= self.start_time
        self.end_time = time

    def set_reserved (self, reserved):
        require(reserved, self.reserved_is)
        self.reserved = reserved

    def set_users (self, users):
        require(users, self.users_is)
        self.users = users

    def get_start_time (self):
        return self.start_time

    def get_end_time (self):
        return self.end_time

    def get_reserved (self):
        return self.reserved

    def get_users (self):
        return self.users

    def is_active(self, start, end):
        return (self.get_end_time() > start and
                self.get_start_time() < end)

    def can_modify (self, user):
        """(user: User) -> bool
        """
        if user in self.users:
            return True
        if user and user.is_admin():
            return True
        for resource in self.reserved:
            if resource.allows_control(user):
                return True
        return False


class Calendar (DulcineaPersistent, KeyedMap):

    """
    Instance attributes: none
    """

    def __init__(self):
        KeyedMap.__init__(self, Reservation)

    def get_reservations(self):
        return self.get_mapping().values()

    def get_table_map(self, resource, time_increment,
                      start_time, end_time,
                      start_time_of_day, end_time_of_day):
        """(resource:Resource,
            time_increment:DateTimeDelta,
            start_time:DateTime,
            end_time:DateTime,
            start_time_of_day:DateTimeDelta,
            end_time_of_day:DateTimeDelta) -> {
                (day:DateTime, time_of_day:DateTimeDelta) :
                (reservation:Reservation, rowspan:int) | None }

        This calculates information useful for displaying a tabular view
        of the calendar for one resource.
        The time_increment is amount of time for each row of the table.
        The start_time and end_time determine the columns (days) of the table.
        The start_time_of_day and end_time_of_day limit the rows that are shown.

        The function returns a dictionary with an entry for each time block
        that is not vacant.  For the first block of each day for a given
        reservation, the dictionary maps (day, time_of_day) to
        (reservation, rowspan), meaning that this reservation appears first
        on this day at this time of day, and that it extends across rowspan
        time increments.

        The cells of the table that are continuations of a reservation that
        has already appeared above have entries that map (day, time_of_day)
        to None.
        """

        reservations = method_sort(
            [reservation for reservation in self.get_reservations()
             if (reservation.get_start_time() < end_time and
                 reservation.get_end_time() > start_time and
                 resource in reservation.get_reserved())],
            'get_start_time')
        tableau = {}
        def gen_days():
            day = start_time
            while day < end_time:
                yield day
                day = day + 1

        for day in gen_days():
            day_reservations = [
                reservation for reservation in reservations
                if reservation.is_active(day + start_time_of_day,
                                         day + end_time_of_day)]
            time_of_day = start_time_of_day
            while time_of_day < end_time_of_day:
                if day_reservations:
                    reservation = day_reservations[0]
                    period_end = day + time_of_day + time_increment
                    if reservation.get_start_time() < period_end:
                        start = (day, time_of_day)
                        reservation_end = reservation.get_end_time()
                        rows = 1
                        next_time = time_of_day + time_increment
                        while (next_time < end_time_of_day and
                               day + next_time < reservation_end):
                            rows += 1
                            time_of_day = next_time
                            next_time = next_time + time_increment
                            tableau[(day, time_of_day)] = None # mark continuation
                        tableau[start] = (reservation, rows)
                        day_reservations = day_reservations[1:]
                time_of_day = time_of_day + time_increment
        return tableau





"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/survey.py $
$Id: survey.py 23466 2004-02-13 19:12:31Z dbinger $
"""

from mx import DateTime
from dulcinea.keyed import Keyed, KeyedMap
from dulcinea.base import DulcineaBase, DulcineaPersistent
from dulcinea.persistent_list import PersistentList
from dulcinea.spec import any, sequence
from dulcinea.typeutils import typecheck, typecheck_seq
from dulcinea.user import DulcineaUser


class Question(Keyed, DulcineaPersistent):
    """
    Instance attributes:
      title : string
        text of the question
      sub_title : string
        sub title
    """

    title_is = any(str, None)
    sub_title_is = any(str, None)

    def __init__(self, title, sub_title=None):
        assert self.__class__ is not Question, "abstract class"
        Keyed.__init__(self)
        self.title = title
        self.sub_title = sub_title

    def get_title(self):
        return self.title

    def get_sub_title(self):
        return self.sub_title

    def get_new_answer(self):
        raise NotImplementedError, "abstract method"


class TextQuestion(Question):

    """
    Instance attributes: none
    """
    def get_new_answer(self):
        return TextAnswer(self)


class SelectQuestion(Question):
    """
    Instance attributes:
      column_headings : [ string ]
        text to place above each column
      row_headings : [ string ]
        text that preceeds each row
      range_labels : [ string ]
        labels to be placed in a row above column headings at the right and left
      last_row_has_text : boolean
        flag to tell the widget to display a TextWidget after the last row label

    A SelectQuestion is an abstract class for representing selections
    from either checkboxes or radio buttons
    """

    column_headings_is = [ any(str, None) ]
    row_headings_is = [ any (str, None) ]
    range_labels_is = [ str ]
    last_row_has_text_is = bool

    def __init__(self, title, sub_title=None,
                 column_headings=None, row_headings=None,
                 range_labels=None, last_row_has_text=False):
        Question.__init__(self, title, sub_title=sub_title)
        self.column_headings = column_headings or []
        self.row_headings = row_headings or []
        self.range_labels = range_labels or []
        assert len(self.range_labels) in (0,2), "must have two range labels"
        self.last_row_has_text = last_row_has_text

    def get_row_headings(self):
        return self.row_headings

    def get_column_headings(self):
        return self.column_headings

    def get_range_labels(self):
        return self.range_labels

    def get_new_answer(self):
        return SelectAnswer(self)



class QuestionDatabase(KeyedMap):
    """
    Instance attributes: none
    """
    def __init__(self):
        KeyedMap.__init__(self, value_spec=Question)

    def get_answers(self, question_keys):
        """(question_keys: [int]) -> [Answer]
        """
        return [self.get(question_key).get_new_answer()
                for question_key in question_keys]

class Answer(DulcineaBase):
    """
    Instance attributes:
      question : Question
        the question associated with this answer
    """

    question_is = Question

    def __init__(self, question):
        assert self.__class__ is not Answer, "abstract class"
        self.question = question

    def get_question(self):
        return self.question


class TextAnswer(Answer):
    """
    Instance attributes:
      answer : string
        text answer to a question
    """

    answer_is = str

    def __init__(self, question):
        Answer.__init__(self, question)
        self.answer = None

    def get_answer(self):
        return self.answer

    def set_answer(self, answer):
        self.answer = answer

    def has_answer(self):
        return self.answer is not None

class SelectAnswer(Answer):
    """
    Instance attributes:
      selected : [ string | boolean ]
        list of selected row answers
      other_text : string
        contents of the filled in 'last_row_has_text' textfield
    """
    selected_is = [any(str, bool)]

    def __init__(self, question):
        Answer.__init__(self, question)
        self.selected = None
        self.other_text = None

    def set_other_text(self, other_text):
        self.other_text = other_text

    def get_other_text(self):
        return self.other_text

    def set_answer(self, selected):
        self.selected = selected

    def get_answer(self):
        return self.selected

    def has_answer(self):
        return self.selected is not None


class Survey(DulcineaPersistent):
    """A set of Questions and Answers attached to a single object.

    Instance attributes:
      user : DulcineaUser
        user taking the survey
      timestamp : DateTime
        set when the Answers are added
      answers : [ Answer ]
        answers to the questions that comprise the object's survey
    """

    user_is = any(DulcineaUser, None)
    timestamp_is = DateTime.DateTimeType
    answers_is = [Answer]

    def __init__(self, answers):
        """(answers:[Answer])
        """
        typecheck_seq(answers, Answer)
        self.answers = answers
        self.timestamp = None
        self.user = None

    def __str__(self):
        return "By %s on %s: %d answers" % (
            self.user, self.timestamp, len(self.answers))

    def __len__(self):
        return len(self.answers)

    def get_questions(self):
        return [answer.get_question() for answer in self.answers]

    def get_answers(self):
        return self.answers

    def get_timestamp(self):
        return self.timestamp

    def get_user(self):
        return self.user

    def set_timestamp(self):
        self.timestamp = DateTime.now()

    def set_user(self, user):
        """(user : DuclineaUser)
        """
        typecheck(user, DulcineaUser, allow_none=1)
        self.user = user

    def set_answers(self, answers, user):
        """(answers : [Answer], user : DulcineaUser)
        """
        typecheck_seq(answers, Answer)
        self.answers = answers
        self.set_user(user)
        self.set_timestamp()

    def has_answers(self):
        """() -> boolean

        Return true if any answers have been answered for this survey
        """
        for answer in self.answers:
            if answer.has_answer():
                return True
        return False


class Surveyable:
    """A mixin for objects that have Surveys

    Instance attributes:
      surveys : PersistentList [ Survey ]
    """

    surveys_is = any([Survey], None)

    def __init__(self):
        self.surveys = None

    def get_surveys(self):
        """() -> [Survey]
        """
        return self.surveys or []

    def get_survey(self, index):
        return self.surveys and self.surveys[index]

    def get_new_survey(self):
        """() -> Survey
        """
        raise NotImplementedError, "abstract method"

    def add_survey(self, survey):
        if self.surveys is None:
            self.surveys = PersistentList()
        self.surveys.append(survey)



"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/attachable.py $
$Id: attachable.py 23118 2003-11-10 22:13:29Z dbinger $

Provides the Attachable mixin class that allows objects to to have
file attachments.
"""

from mx import DateTime
from dulcinea.typeutils import typecheck
from dulcinea.stored_file import StoredFile

class Attachment:
    """
    Stores properties of an attachment relationship between an Attachable
    object and a StoredFile.

    Instance attributes:
      file : StoredFile
        the file attached
      owner : User
        the owner of this connection object
      date : DateTime
        date this attachment was created
      filename : string
        the filename to use when downloading the file
      description : string
        a description of the file
    """

    def __init__(self, file, owner):
        typecheck(file, StoredFile)
        self.file = file
        self.owner = owner
        self.date = DateTime.now()
        self.filename = file.get_filename()
        self.description = file.get_description()

    def get_file(self):
        return self.file

    def open(self):
        return self.file.open()

    def get_owner(self):
        return self.owner

    def get_date(self):
        return self.date

    def get_filename(self):
        return self.filename

    def set_filename(self, filename):
        self.filename = filename

    def get_description(self):
        return self.description

    def set_description(self, description):
        self.description = description

    def get_size(self):
        return self.file.get_size()

    def get_mime_type(self):
        return self.file.get_mime_type()

    def has_manage_access(self, user):
        return (user and (user is self.owner or user.is_admin()))


class Attachable:
    """
    Instance attributes:
      attachments : [ Attachment ]
        attachments connected to this object

    """
    def __init__(self, attachments=None):
        if attachments:
            self.attachments = list(attachments)
        else:
            self.clear_attachments()

    def clear_attachments (self):
        self.attachments = []

    def get_attachments (self):
        """() -> [Attachment]
        Retrieve a list of the attachments for this object.
        """
        return self.attachments

    def get_attached_files (self):
        """() -> [StoredFile]
        Retrieve a list of the file objects attached to this object.
        """
        return [a.get_file() for a in self.attachments]

    def attach_file (self, stored_file, user):
        """(stored_file: StoredFile)
        Attach a file to this object.
        """
        typecheck(stored_file, StoredFile)
        files = self.attachments[:]
        files.append(Attachment(stored_file, user))
        self.attachments = files

    def detach_attachment (self, attachment):
        """(attachment: Attachment)
        Detach an attachment from this object.
        """
        typecheck(attachment, Attachment)
        files = self.attachments[:]
        files.remove(attachment)
        self.attachments = files

    def attachment_modified(self, attachment, user):
        """(attachment : Attachment, user : DulcineaUser)

        'attachment' was modified by 'user'.
        Override this method to intercept modification activity for attachments
        """
        pass

    def get_allowed_mime_types(self):
        """() -> [string]

        Override to restrict the types of files that can be attached
        """
        return [mime_type[0] for mime_type in StoredFile.MIME_TYPES]





#!/www/python/bin/python
"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/test/test_user_database.py $
$Id: test_user_database.py 23436 2004-02-05 18:39:13Z dbinger $
"""

from sancho.unittest import TestScenario, parse_args, run_scenarios
from dulcinea.errors import ConflictError
from dulcinea.user import DulcineaUser
from dulcinea.user_database import DulcineaUserDatabase, ContactDatabase

tested_modules = ["dulcinea.user_database"]

class Test(TestScenario):

    def check_user_database(self):
        user1 = DulcineaUser('1')
        user2 = DulcineaUser('2')
        self.test_stmt("self.db = DulcineaUserDatabase()")
        self.test_stmt("self.db.add_user(user1)")
        self.test_val("self.db.get_user_ids()", ['1'])
        self.test_val("self.db.get_users()", [user1])
        self.test_stmt("self.db.add_user(user2)")
        self.test_val("self.db.get_users(sort=True)", [user1, user2])
        self.test_val("self.db.get_user('1')", user1)
        self.test_val("self.db['1']", user1)
        self.test_exc("self.db.add_user(user1)", ConflictError)
        self.test_val("self.db.get_admin()", None)

    def check_contact_database(self):
        self.test_stmt("self.db = ContactDatabase()")
        self.test_val("str(self.db.get_admin())", 'admin')

if __name__ == "__main__":
    (scenarios, options) = parse_args()
    run_scenarios (scenarios, options)
